# -*- coding: utf-8 -*-
import sys
import numpy as np
import os
import glob
import pandas as pd
import cv2  # type: ignore
import torch
import shutil
import tempfile
import uuid
from PyQt5 import QtWidgets,QtCore,QtGui
from PyQt5.QtWidgets import *
from PyQt5.QtGui import QImage, QPixmap, QPainterPath, QPainter, QPen, QColor
from PyQt5.QtCore import Qt, QPointF, QPoint, QRectF

current_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, current_dir)
sys.path.append('/segment_anything')
from segment_anything import SamPredictor, sam_model_registry
import logging

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(levelname)s - %(message)s"
)
log = logging.getLogger(__name__)

sys.path.append('/functions')
from functions.inverted_colors_func import inverted_colors_func
from functions.grayscale_func import grayscale_func
from functions.enhancement_func import enhancement_func
from functions.binarize_func import binarize_func
from functions.corrosion_func import corrosion_func
from functions.dilation_func import dilation_func
from functions.gui_chain_code_func import gui_chain_code_func
from functions.contour_extraction_func import contour_extraction_func
from functions.cal_area_c import cal_area_c
from functions.is_completed_chain_code import is_completed_chain_code
from functions.calc_traversal_dist import calc_traversal_dist
from functions.write_Data_To_File import write_data_to_file
from functions.EFA import MainWindow_1 


class CustomGraphicsView(QGraphicsView):
    def __init__(self, scene, main_window,*args, **kwargs):
        super(CustomGraphicsView, self).__init__(main_window,*args, **kwargs)
        self.setScene(scene)
        self.drawing = False
        self.last_point = QPoint()
        self.main_window = main_window
        self.pixmap_result_item = None  
        self.drawn_paths = []  # 存储所有绘制的路径项
        self.show_sam_result()
    
    def show_sam_result(self):
        self.scene().clear() 
        # print(self.main_window.bin)
        if hasattr(self.main_window, 'bin') and len(self.main_window.bin) > 0:  # 动态获取 bin
            pixmap_result = self.numpy_to_pixmap(self.main_window.bin)
            self.pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
            self.scene().addItem(self.pixmap_result_item)

    def mousePressEvent(self, event):
        try:
            items = self.scene().items()
            log.info("%s", len(items))
            if len(items) > 1:
                for item in items[:-1]:
                    if  isinstance(item, QGraphicsLineItem):
                        pen = item.pen()
                        try:
                            if pen.color() == QColor(255, 0, 0) or pen.color() == QColor(0, 255, 0):
                                # print("CustomGraphicsView")
                                self.scene().removeItem(item)
                        except ValueError as e:
                            QMessageBox.warning(self,'Error','{e}')
                            return
            pos = event.pos()
            pos_adjusted = self.mapToScene(pos).toPoint()
            self.last_point = pos_adjusted
            if event.button() == Qt.LeftButton:
                self.drawing= True
            if event.button() == Qt.RightButton:
                self.show_context_menu(event)
            log.info(f"Mouse Press at {pos_adjusted}")
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return
    def mouseMoveEvent(self, event):
        try:
            if self.drawing:
                current_pos = self.mapToScene(event.pos())
                # 动态获取当前像素的 bin 值以决定画笔颜色
                x = int(current_pos.x())
                y = int(current_pos.y())
                if 0 <= x < self.main_window.bin.shape[1] and 0 <= y < self.main_window.bin.shape[0]:
                    current_value = self.main_window.bin[y, x]
                    pen_color = QColor(255, 255, 255) if current_value == 0 else QColor(0, 0, 0)
                else:
                    pen_color = QColor(255, 255, 255)  # 默认白色

                # 创建路径项并绘制线段
                path = QPainterPath()
                path.moveTo(self.last_point)
                path.lineTo(current_pos)
                path_item = QGraphicsPathItem(path)
                path_item.setPen(QPen(pen_color, 1))
                self.scene().addItem(path_item)
                self.drawn_paths.append(path_item)  # 保存路径项

                self.last_point = current_pos
                log.info(f"Mouse Move to {current_pos}")
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return
    def mouseReleaseEvent(self, event):
        try:
            if event.button() == Qt.LeftButton:
                self.drawing = False
                log.info("Mouse Release")
                # self.save_scene_as_image()
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return
    def draw_line(self, start_point, end_point,pen):
        try:
            line = self.scene().addLine(start_point.x(), start_point.y(), end_point.x(), end_point.y(),pen)
            log.info(f"Draw line from {start_point} to {end_point}")
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def show_context_menu(self, event):
        context_menu = QMenu(self)
        reset_action = QAction("Reset", self)
        reset_action.triggered.connect(self.reset_scene)
        context_menu.addAction(reset_action)
        context_menu.exec_(self.mapToGlobal(event.pos()))

    def reset_scene(self):
        self.show_sam_result()
        log.info("Scene has been reset")
        QMessageBox.information(self, "Reset", "All edits have been reset")

    def numpy_to_pixmap(self, numpy_array):
        try:
            if len(numpy_array.shape) == 2:  
                height, width = numpy_array.shape
                bytes_per_line = width
                q_image = QImage(numpy_array.data, width, height, bytes_per_line, QImage.Format_Grayscale8)
                return QPixmap.fromImage(q_image)
            elif len(numpy_array.shape) == 3:  
                height, width, channel = numpy_array.shape
                bytes_per_line = channel * width
                if channel == 3:
                    q_image = QImage(numpy_array.data, width, height, bytes_per_line, QImage.Format_RGB888)
                elif channel == 4:
                    q_image = QImage(numpy_array.data, width, height, bytes_per_line, QImage.Format_RGBA8888)
                else:
                    raise ValueError("Unsupported number of channels: {}".format(channel))
                return QPixmap.fromImage(q_image)
            else:
                raise ValueError("Unsupported image format: {}".format(numpy_array.shape))
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return



class CustomGraphicsView_1(QGraphicsView):  
    def __init__(self, main_window, *args):
        super().__init__( main_window.scene, main_window, *args)
        self.setMouseTracking(True)  
        
        self.main_window = main_window  
        self.setScene(main_window.scene)
        self.scene_1 = self.main_window.scene
        
        self.start_point = None  
        self.end_point = None
        self.current_rect_item = None  
        self.is_drawing = False  
        
        self.polygon = []
        
        self.scene_2 = self.main_window.scene_2
        self.graphicsView_2 = self.main_window.graphicsView_2
        self.pushButton_8 = self.main_window.pushButton_8
        
        self.points_pre = []
        self.points = []
        
        self.polygon_item = None

    def mousePressEvent(self, event):

        if event.button() == Qt.LeftButton and self.main_window.flag_btn_6==0:
            if self.main_window.image:
                self.scene_1.clear()
                pixmap_item = QGraphicsPixmapItem(self.main_window.image)
                self.scene_1.addItem(pixmap_item)
            self.is_drawing = True
        
            self.start_point = self.mapToScene(event.pos())
            
            self.current_rect_item = QGraphicsRectItem()
            self.current_rect_item.setPen(Qt.red)
            self.scene_1.addItem(self.current_rect_item)
        
        try:
            print(self.main_window.flag_btn_6)
            if (self.main_window.flag_btn_6==1) & (event.buttons() == Qt.LeftButton) : 
                # print("执行画线")   
                scenepos = self.mapToScene(event.pos())          
                
                log.info("Mouse Position (Scene Coordinates): (%s, %s)", scenepos.x(),scenepos.y())
                # print("Mouse Position (View Coordinates):", pos_adjusted.x(), pos_adjusted.y())
                # print("Mouse Position (Scene Coordinates):", self.scene_pos[0],  self.scene_pos[1])
                self.polygon.append(QPointF(scenepos.x(),scenepos.y()))
                print(self.polygon)
                self.updatePolygon()
                    

            elif (self.main_window.flag_btn_6==1) & (event.buttons() == Qt.RightButton) :
                pixmap_item=self.cropImage()          
                height=self.main_window.image.height()
                width=self.main_window.image.width()   
                mask = np.zeros((height, width), dtype=np.uint8)
                if self.polygon == None:
                    return
                print(self.polygon[0])
                self.scene_1.addLine(self.polygon[0].x(), self.polygon[0].y(), self.polygon[-1].x(), self.polygon[-1].y(), QPen(Qt.red,3))
                pts = np.array([(point.x(), point.y()) for point in self.polygon], dtype=np.int32)
                
                
                cv2.fillPoly(mask, [pts], 255)
                # print(np.shape(self.main_window.image))
                try:
                    img_num=self.main_window.pixmap_to_numpy(self.main_window.image)
                except ValueError as e:
                    QMessageBox.warning(self,'Error','{e}')
                    return
                result = cv2.bitwise_and(img_num, img_num, mask=mask)
                result[(result[:,:,0] == 0) & (result[:,:,1] == 0) & (result[:,:,2] == 0)] = [255, 255, 255]
                # cv2.imwrite('result.png',result)
                self.main_window.crop_result =  cv2.cvtColor(result, cv2.COLOR_BGR2RGB)
                pixmap_result = self.main_window.numpy_to_pixmap(self.main_window.crop_result)
                pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
                self.scene_2.clear()
                self.scene_2.addItem(pixmap_result_item)
                self.graphicsView_2.setScene(self.scene_2)
                self.graphicsView_2.fitInView(self.graphicsView_2.sceneRect(), Qt.KeepAspectRatio)
                self.polygon = []
                self.polygon_item = None
                self.main_window.flag_btn_6=0
                self.pushButton_8.setEnabled(True)

            if (self.main_window.flag_point == 1) & (event.button() == Qt.LeftButton) :           
                if hasattr(self, 'points'):
                    self.points_pre.append((event.x(), event.y()))
                else:
                    self.points_pre = [(event.x(), event.y())]
                # print(self.points_pre)
                # print(len(self.points_pre))
                if len(self.points_pre) == 2:
                    pos_dis1 = QtCore.QPoint(self.points_pre[0][0], self.points_pre[0][1])
                    scenepos_dis1 = self.mapToScene(pos_dis1)
                    self.points.append((scenepos_dis1.x(),scenepos_dis1.y()))
                    pos_dis2 = QtCore.QPoint(self.points_pre[1][0], self.points_pre[1][1])
                    scenepos_dis2 = self.mapToScene(pos_dis2)
                    self.points.append((scenepos_dis2.x(),scenepos_dis2.y()))
                    # print(self.points)
                    self.main_window.distance = ((self.points[1][0] - self.points[0][0])**2 + (self.points[1][1] - self.points[0][1])**2)**0.5

                    point_item = QGraphicsEllipseItem(self.points[0][0], self.points[0][1]-10, 20, 20)
                    point_item.setBrush(Qt.red)  
                    self.scene_1.addItem(point_item)
                    point_item = QGraphicsEllipseItem(self.points[1][0]-10, self.points[1][1]-10, 20, 20)  
                    point_item.setBrush(Qt.red)  
                    self.scene_1.addItem(point_item) 

                    # self.graphicsView.setScene(self.scene)
                    self.points_pre=[]
                    self.points = []
                    self.main_window.flag_point=0
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return
        # super().mousePressEvent(event)

    def mouseMoveEvent(self, event):
        try:
            # print(self.start_point)
            if self.is_drawing and self.start_point:
                # print('movement')
                
                self.end_point = self.mapToScene(event.pos())
                
                rect = QRectF(self.start_point, self.end_point).normalized()  # 归一化确保矩形方向正确
                self.current_rect_item.setRect(rect)
            # super().mouseMoveEvent(event)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return
        
        

    def mouseReleaseEvent(self, event):
        try:
            if event.button() == Qt.LeftButton and self.is_drawing:
                
                self.is_drawing = False
                # self.start_point = None
                # self.pointsChanged.emit(self.start_point, self.end_point)  # 触发信号
                print("Rectangle drawn:", self.current_rect_item.rect())
                # Ensure self.polygon and self.image are valid
                if not self.current_rect_item.rect().isEmpty():
                    height=self.main_window.image.height()
                    width=self.main_window.image.width()  
                    mask = np.zeros((height, width), dtype=np.uint8)

                    # 提取矩形四个顶点作为多边形点
                    rect = self.current_rect_item.rect()
                    top_left = (rect.x(), rect.y())
                    top_right = (rect.x() + rect.width(), rect.y())
                    bottom_right = (rect.x() + rect.width(), rect.y() + rect.height())
                    bottom_left = (rect.x(), rect.y() + rect.height())
                    polygon_points = np.array([top_left, top_right, bottom_right, bottom_left], dtype=np.int32)
                    
                    cv2.fillPoly(mask, [polygon_points], 255)
                    # print(np.shape(self.main_window.image))
                    try:
                        img_num=self.main_window.pixmap_to_numpy(self.main_window.image)
                    except ValueError as e:
                        QMessageBox.warning(self,'Error','{e}')
                        return
                    result = cv2.bitwise_and(img_num, img_num, mask=mask)
                    result[(result[:,:,0] == 0) & (result[:,:,1] == 0) & (result[:,:,2] == 0)] = [255, 255, 255]
                    # cv2.imwrite('result.png',result)
                    self.main_window.crop_result =  cv2.cvtColor(result, cv2.COLOR_BGR2RGB)
                    pixmap_result = self.main_window.numpy_to_pixmap(self.main_window.crop_result)
                    pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
                    self.scene_2.clear()
                    self.scene_2.addItem(pixmap_result_item)
                    self.graphicsView_2.setScene(self.scene_2)
                    self.graphicsView_2.fitInView(self.graphicsView_2.sceneRect(), Qt.KeepAspectRatio)

            # super().mouseReleaseEvent(event)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return
    def updatePolygon(self):
        try:
            # if self.polygon_item:
            #     print("polygon item")
            #     self.polygon_item=None
            #     # self.scene_1.removeItem(self.polygon_item)
            #     print("after polygon item")
            if len(self.polygon) < 2:
                return
            path = QPainterPath()
            # print(self.polygon[0])
            path.moveTo(self.polygon[0])
            for point in self.polygon[1:]:
                path.lineTo(point)
            # path.lineTo(self.polygon[0])
            self.polygon_item = self.scene_1.addPath(path, QPen(Qt.red,3))
            self.polygon_item=None
            # print("done")
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def cropImage(self):
        try:
            if len(self.polygon) < 3:
                return
            path = QPainterPath()
            path.moveTo(self.polygon[0])
            for point in self.polygon[1:]:
                path.lineTo(point)
            path.lineTo(self.polygon[0])
            region = QImage(self.main_window.image.size(), QImage.Format_ARGB32)
            region.fill(Qt.transparent)
            painter = QPainter(region)
            painter.setRenderHint(QPainter.Antialiasing)
            painter.setClipPath(path)
            # painter.drawImage(0, 0, self.image)
            painter.end()
            cropped_pixmap = QPixmap.fromImage(region)
            pixmap_item = QGraphicsPixmapItem(cropped_pixmap)
            return pixmap_item
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return
    

class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setObjectName("MainWindow")
        self.resize(1795, 950)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.setFont(font)
        self.centralwidget = QtWidgets.QWidget(self)
        self.centralwidget.setObjectName("centralwidget")


        self.groupBox_4 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_4.setGeometry(QtCore.QRect(10, 5, 591, 65))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.groupBox_4.setFont(font)
        self.groupBox_4.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.groupBox_4.setObjectName("groupBox_4")


        self.pushButton = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton.setGeometry(QtCore.QRect(12, 30, 101, 35))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.pushButton.setFont(font)
        self.pushButton.setObjectName("pushButton")

        self.graphicsView_2 = QtWidgets.QGraphicsView(self.centralwidget)
        self.graphicsView_2.setGeometry(QtCore.QRect(610, 75, 481, 321))
        self.graphicsView_2.setObjectName("graphicsView_2")
        self.graphicsView_3 = QtWidgets.QGraphicsView(self.centralwidget)
        self.graphicsView_3.setGeometry(QtCore.QRect(610, 405, 481, 331))
        self.graphicsView_3.setObjectName("graphicsView_3")
        

        self.scene_3 = QGraphicsScene(self)
        self.bin = []
        self.graphicsView_4 = CustomGraphicsView(self.scene_3,self)
        self.graphicsView_4.setGeometry(QtCore.QRect(1100, 75, 681, 661))
        self.graphicsView_4.setObjectName("graphicsView_4")
        self.comboBox = QtWidgets.QComboBox(self.centralwidget)
        self.comboBox.setGeometry(QtCore.QRect(120, 30, 317, 35))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.comboBox.setFont(font)
        self.comboBox.setObjectName("comboBox")


        self.pushButton_2 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_2.setGeometry(QtCore.QRect(447, 30, 71, 35))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(18)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.pushButton_2.setFont(font)
        self.pushButton_2.setObjectName("pushButton_2")
        self.pushButton_3 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_3.setGeometry(QtCore.QRect(528, 30, 71, 35))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(18)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.pushButton_3.setFont(font)
        self.pushButton_3.setObjectName("pushButton_3")


        self.groupBox_5 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_5.setGeometry(QtCore.QRect(610, 5, 481, 65))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.groupBox_5.setFont(font)
        self.groupBox_5.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.groupBox_5.setObjectName("groupBox_5")

        self.label_6 = QtWidgets.QLabel(self.centralwidget)
        self.label_6.setGeometry(QtCore.QRect(842, 30, 20, 35))
        self.label_6.setObjectName("label_6")

        self.pushButton_5 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_5.setGeometry(QtCore.QRect(612, 30, 215, 35))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.pushButton_5.setFont(font)
        self.pushButton_5.setObjectName("pushButton_5")
        self.pushButton_6 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_6.setGeometry(QtCore.QRect(874, 30, 215, 35)) 
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.pushButton_6.setFont(font)
        self.pushButton_6.setObjectName("pushButton_6")


        self.groupBox_6 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_6.setGeometry(QtCore.QRect(1100, 5, 681, 65))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.groupBox_6.setFont(font)
        self.groupBox_6.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.groupBox_6.setObjectName("groupBox_6")

        self.pushButton_4 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_4.setGeometry(QtCore.QRect(1560, 30, 219, 35))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.pushButton_4.setFont(font)
        self.pushButton_4.setObjectName("pushButton_4")

        self.pushButton_7 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_7.setGeometry(QtCore.QRect(1331, 30, 219, 35))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.pushButton_7.setFont(font)
        self.pushButton_7.setObjectName("pushButton_7")
        self.pushButton_8 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_8.setGeometry(QtCore.QRect(1102, 30, 219, 35))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.pushButton_8.setFont(font)
        self.pushButton_8.setObjectName("pushButton_8")



        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox.setGeometry(QtCore.QRect(905, 740, 876, 120))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.groupBox.setFont(font)
        self.groupBox.setObjectName("groupBox")
        self.pushButton_9 = QtWidgets.QPushButton(self.groupBox)
        self.pushButton_9.setGeometry(QtCore.QRect(10, 25, 200, 35))
        self.pushButton_9.setObjectName("pushButton_9")
        self.textEdit_5 = QtWidgets.QTextEdit(self.groupBox)
        self.textEdit_5.setGeometry(QtCore.QRect(360, 25, 281, 35))
        self.textEdit_5.setObjectName("textEdit_5")
        self.radioButton = QtWidgets.QRadioButton(self.groupBox)
        self.radioButton.setGeometry(QtCore.QRect(670, 25, 51, 16))
        self.radioButton.setObjectName("radioButton")
        self.radioButton.setChecked(True)
        self.buttonGroup = QtWidgets.QButtonGroup(self)
        self.buttonGroup.setObjectName("buttonGroup")
        self.buttonGroup.addButton(self.radioButton)
        self.radioButton_2 = QtWidgets.QRadioButton(self.groupBox)
        self.radioButton_2.setGeometry(QtCore.QRect(670, 43, 61, 16))
        self.radioButton_2.setObjectName("radioButton_2")
        self.buttonGroup.addButton(self.radioButton_2)
        self.pushButton_10 = QtWidgets.QPushButton(self.groupBox)
        self.pushButton_10.setGeometry(QtCore.QRect(741, 25, 125, 35))
        self.pushButton_10.setObjectName("pushButton_10")
        self.label_5 = QtWidgets.QLabel(self.groupBox)
        self.label_5.setGeometry(QtCore.QRect(220, 25, 150, 35))
        self.label_5.setObjectName("label_5")
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setGeometry(QtCore.QRect(50, 75, 51, 35))
        self.label_4.setObjectName("label_4")
        self.textEdit_7 = QtWidgets.QTextEdit(self.groupBox)
        self.textEdit_7.setGeometry(QtCore.QRect(110, 75, 100, 35))
        self.textEdit_7.setAutoFillBackground(False)
        self.textEdit_7.setMidLineWidth(0)
        self.textEdit_7.setObjectName("textEdit_7")
        self.pushButton_16 = QtWidgets.QPushButton(self.groupBox)
        self.pushButton_16.setGeometry(QtCore.QRect(220, 75, 646, 35))
        self.pushButton_16.setObjectName("pushButton_16")


        self.pushButton_17 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_17.setGeometry(QtCore.QRect(10, 870, 1771, 50))
        self.pushButton_17.setIconSize(QtCore.QSize(30, 30))
        self.pushButton_17.setObjectName("pushButton_17")


        
        self.groupBox_2 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_2.setGeometry(QtCore.QRect(10, 740, 885, 120))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.groupBox_2.setFont(font)
        self.groupBox_2.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.groupBox_2.setObjectName("groupBox_2")
        self.radioButton_3 = QtWidgets.QRadioButton(self.groupBox_2)
        self.radioButton_3.setGeometry(QtCore.QRect(20, 30, 211, 19))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.radioButton_3.setFont(font)
        self.radioButton_3.setObjectName("radioButton_3")
        self.radioButton_3.setChecked(True)
        self.radioButton_3.setEnabled(False)
        self.buttonGroup_2 = QtWidgets.QButtonGroup(self)
        self.buttonGroup_2.setObjectName("buttonGroup_2")
        self.buttonGroup_2.addButton(self.radioButton_3)
        self.radioButton_4 = QtWidgets.QRadioButton(self.groupBox_2)
        self.radioButton_4.setGeometry(QtCore.QRect(20, 80, 211, 19))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.radioButton_4.setFont(font)
        self.radioButton_4.setObjectName("radioButton_4")
        self.radioButton_4.setEnabled(False)
        self.buttonGroup_2.addButton(self.radioButton_4)
        self.pushButton_11 = QtWidgets.QPushButton(self.groupBox_2)
        self.pushButton_11.setGeometry(QtCore.QRect(220, 25, 100, 35))
        self.pushButton_11.setObjectName("pushButton_11")
        self.pushButton_12 = QtWidgets.QPushButton(self.groupBox_2)
        self.pushButton_12.setGeometry(QtCore.QRect(475, 25, 100, 35))
        self.pushButton_12.setObjectName("pushButton_12")
        self.label_3 = QtWidgets.QLabel(self.groupBox_2)
        self.label_3.setGeometry(QtCore.QRect(345, 25, 40, 35))
        self.label_3.setObjectName("label_3")
        self.textEdit_6 = QtWidgets.QTextEdit(self.groupBox_2)
        self.textEdit_6.setGeometry(QtCore.QRect(395, 25, 70, 35))
        self.textEdit_6.setAutoFillBackground(False)
        self.textEdit_6.setMidLineWidth(0)
        self.textEdit_6.setObjectName("textEdit_6")
        self.pushButton_13 = QtWidgets.QPushButton(self.groupBox_2)
        self.pushButton_13.setGeometry(QtCore.QRect(585, 25, 150, 35))
        self.pushButton_13.setObjectName("pushButton_13")
        self.comboBox_2 = QtWidgets.QComboBox(self.groupBox_2)
        self.comboBox_2.setGeometry(QtCore.QRect(220, 75, 100, 35))
        self.comboBox_2.setObjectName("comboBox_2")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("canny")
        self.comboBox_2.addItem("sobel")
        self.comboBox_2.addItem("zerocross")
        self.comboBox_2.addItem("laplace")
        self.comboBox_2.addItem("Roberts")
        self.comboBox_2.addItem("Prewitt")
        self.horizontalSlider = QtWidgets.QSlider(self.groupBox_2)
        self.horizontalSlider.setGeometry(QtCore.QRect(345, 75, 230, 35))
        self.horizontalSlider.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSlider.setObjectName("horizontalSlider")
        self.pushButton_14 = QtWidgets.QPushButton(self.groupBox_2)
        self.pushButton_14.setGeometry(QtCore.QRect(585, 75, 150, 35))
        self.pushButton_14.setObjectName("pushButton_14")
        self.pushButton_15 = QtWidgets.QPushButton(self.groupBox_2)
        self.pushButton_15.setGeometry(QtCore.QRect(745, 25, 130, 85))
        self.pushButton_15.setObjectName("pushButton_15")


        self.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(self)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1795, 25))
        self.menubar.setObjectName("menubar")
        self.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(self)
        self.statusbar.setObjectName("statusbar")
        self.setStatusBar(self.statusbar)
        _translate = QtCore.QCoreApplication.translate
        self.setWindowTitle(_translate("MainWindow", "ElliShape"))
        self.pushButton.setText(_translate("MainWindow", "Folder"))
        self.pushButton_2.setText(_translate("MainWindow", "→"))
        self.pushButton_3.setText(_translate("MainWindow", "←"))
        self.pushButton_4.setText(_translate("MainWindow", "Inverted Colors"))
        self.pushButton_5.setText(_translate("MainWindow", "Automated Segmentation"))
        self.pushButton_6.setText(_translate("MainWindow", "Polygon Tool"))
        self.pushButton_7.setText(_translate("MainWindow", "Image Enhancement"))
        self.pushButton_8.setText(_translate("MainWindow", "Grayscale Conversion"))
        self.groupBox.setTitle(_translate("MainWindow", "4 Measurement and save"))
        self.groupBox_4.setTitle(_translate("MainWindow", "1 Input"))
        self.groupBox_5.setTitle(_translate("MainWindow", "2.1 Extraction"))
        self.groupBox_6.setTitle(_translate("MainWindow", "2.2 Image processing"))
        self.pushButton_9.setText(_translate("MainWindow", "Click two points"))

        self.textEdit_5.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Arial\',\'Arial\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.radioButton.setText(_translate("MainWindow", "mm"))
        self.radioButton_2.setText(_translate("MainWindow", "inch"))
        self.pushButton_10.setText(_translate("MainWindow", "Skip"))
        self.label_5.setText(_translate("MainWindow", "Actual distance:"))
        self.label_6.setText(_translate("MainWindow", "or"))

        self.groupBox_2.setTitle(_translate("MainWindow", "3 Chain code"))
        self.radioButton_3.setText(_translate("MainWindow", "Image Binarization"))
        
        self.radioButton_4.setText(_translate("MainWindow", "Edge Detection"))
        self.pushButton_11.setText(_translate("MainWindow", "Binarization"))
        self.pushButton_12.setText(_translate("MainWindow", "Erosion"))
        self.label_3.setText(_translate("MainWindow", "Size:"))
        self.textEdit_6.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Arial\',\'Arial\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Arial\';\">3</span></p></body></html>"))
        self.pushButton_13.setText(_translate("MainWindow", "Dilation "))
        self.comboBox_2.setItemText(0, _translate("MainWindow", "Select"))
        self.pushButton_14.setText(_translate("MainWindow", "Contour Extraction"))
        self.pushButton_15.setText(_translate("MainWindow", "Chain Code"))

        self.label_4.setText(_translate("MainWindow", "Tag:"))
        self.textEdit_7.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Arial\',\'Arial\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Arial\';\">1</span></p></body></html>"))
        self.pushButton_16.setText(_translate("MainWindow", "Save (chain codes, labeled images and size)"))
        self.pushButton_17.setText(_translate("MainWindow", "Elliptic Fourier Analysis"))
        self.folder_path = ''
        self.cwd = os.getcwd() 
        self.image = []
        self.gray_img = []
        
        self.crop_result = []
        # self.scene_pos=[0,0]
        self.scene_2=QGraphicsScene()
        # self.polygon = []
        
        self.flag_point = 0
        self.flag_btn_6 = 0
        self.is_dragging = False
        # self.points = []
        # self.points_pre = []
        self.boundary = []
        self.chaincode = []
        self.last_point = QPoint()   
        self.drawing = False   
        self.last_point = [] 
        self.id_full = '1'
        self.filename = ''
        self.unit = 1
        self.distance = 0

        self.scene = QGraphicsScene(self)
        self.graphicsView = CustomGraphicsView_1(self)
        self.graphicsView.setGeometry(QtCore.QRect(10, 75, 591, 661))
        self.graphicsView.setObjectName("graphicsView")

        for widget in self.centralWidget().findChildren(QPushButton):
            widget.setDisabled(True)
        self.pushButton.setEnabled(True)
        self.pushButton.clicked.connect(self.on_pushButton_clicked)
        self.comboBox.activated.connect(self.comboBox_Callback)
        self.pushButton_2.clicked.connect(self.Nextbtn_clicked)
        self.pushButton_3.clicked.connect(self.Previousbtn_clicked)       
        self.pushButton_4.clicked.connect(self.Inverted_color)
        self.pushButton_5.clicked.connect(self.SAM_segment)
        self.pushButton_6.clicked.connect(self.ROI_selection)
        self.pushButton_8.clicked.connect(self.grayscale)
        self.pushButton_7.clicked.connect(self.enhancement)
        self.buttonGroup.buttonClicked.connect(self.units_set)
        self.buttonGroup_2.buttonClicked.connect(self.method_set)
        self.pushButton_11.clicked.connect(self.binarization)
        self.pushButton_12.clicked.connect(self.corrosion)
        self.pushButton_13.clicked.connect(self.dilation)
        self.pushButton_15.clicked.connect(self.chain_code)
        self.pushButton_14.clicked.connect(self.functions_selection)
        self.pushButton_9.clicked.connect(self.clicked_point)
        self.pushButton_16.clicked.connect(self.save)
        self.pushButton_10.clicked.connect(self.skip)
        self.pushButton_17.clicked.connect(self.go_to_window2)
        

    def on_pushButton_clicked(self):
        try:
            self.folder_path = QFileDialog.getExistingDirectory(self, "select folder", self.cwd)             
            if self.folder_path:
                self.comboBox.clear()
                self.image_files = []
                for file_path in glob.glob(os.path.join(self.folder_path, "*")):
                    if file_path.lower().endswith((".png", ".jpg", ".jpeg", ".tif", ".tiff")):
                        self.image_files.append(os.path.basename(file_path))
                self.comboBox.addItems(self.image_files)
                log.info("Number of images: %s", len(self.image_files))
            else:
                QMessageBox.warning(self, "Warning", "folder is empty")
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return    


    def comboBox_Callback(self):
        try:
            for widget in self.centralWidget().findChildren(QPushButton):
                widget.setDisabled(True)
            self.pushButton.setEnabled(True)
            self.pushButton_2.setEnabled(True)
            self.pushButton_3.setEnabled(True) 
            self.polygon = []
            self.polygon_item = None
            self.filename = self.comboBox.currentText()
            self.c_index = self.comboBox.currentIndex()
            file_path = os.path.join(self.folder_path,self.filename)
            if self.filename:
                # a=cv2.imread(file_path)
                # cv2.imshow('activated',a)
                # self.load_image(file_path)            
                self.image = QPixmap(file_path) 
                pixmap_item = QGraphicsPixmapItem(self.image)
                # print(pixmap_item)
                self.scene.clear()
                self.scene_2.clear()
                self.scene_3.clear()
                self.scene.addItem(pixmap_item)

                self.scene.setSceneRect(pixmap_item.boundingRect())
                self.graphicsView.setScene(self.scene)
                self.graphicsView.fitInView(self.scene.sceneRect(), Qt.KeepAspectRatio)
                self.graphicsView.update()
                self.graphicsView.viewport().update()

                self.pushButton_2.setEnabled(True)
                self.pushButton_3.setEnabled(True)
                self.pushButton_4.setEnabled(True)
                self.pushButton_5.setEnabled(True)
                self.pushButton_6.setEnabled(True)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return


    def ROI_selection(self):
        self.flag_btn_6=1
        self.scene.clear()
        pixmap_item = QGraphicsPixmapItem(self.image)
        self.scene.addItem(pixmap_item)
        


    def Previousbtn_clicked(self):
        try:
            if self.c_index != 0:
                for widget in self.centralWidget().findChildren(QPushButton):
                    widget.setDisabled(True)
                self.pushButton.setEnabled(True)
                self.pushButton_2.setEnabled(True)
                self.pushButton_3.setEnabled(True) 
                self.c_index -= 1
                # print(self.c_index)
                self.filename = self.image_files[self.c_index]
                file_path = os.path.join(self.folder_path,self.filename)
                # print(file_path)
                self.image = QPixmap(file_path)
                ncols = self.image.width()            
                nrows = self.image.height()
                pixmap_item = QGraphicsPixmapItem(self.image)
                self.scene.clear()
                self.scene_2.clear()
                self.scene_3.clear()
                self.scene.addItem(pixmap_item)
                self.scene.setSceneRect(pixmap_item.boundingRect())
                self.graphicsView.setScene(self.scene)
                self.graphicsView.fitInView(self.scene.sceneRect(), Qt.KeepAspectRatio)

                self.comboBox.setCurrentIndex(self.c_index)
                self.pushButton_2.setEnabled(True)
                self.pushButton_3.setEnabled(True)
                self.pushButton_4.setEnabled(True)
                self.pushButton_5.setEnabled(True)
                self.pushButton_6.setEnabled(True)
            else:
                QMessageBox.warning(self, "Warning", "This is the first one.")
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def Nextbtn_clicked(self):
        try:
            if self.c_index != len(self.image_files)-1:
                for widget in self.centralWidget().findChildren(QPushButton):
                    widget.setDisabled(True)
                self.pushButton.setEnabled(True)
                self.pushButton_2.setEnabled(True)
                self.pushButton_3.setEnabled(True) 
                self.c_index += 1
                # print(self.c_index)
                self.filename = self.image_files[self.c_index]
                file_path = os.path.join(self.folder_path,self.filename)
                # print(file_path)
                self.image = QPixmap(file_path)
                ncols = self.image.width()            
                nrows = self.image.height()
                pixmap_item = QGraphicsPixmapItem(self.image)
                self.scene.clear()
                self.scene_2.clear()
                self.scene_3.clear()
                self.scene.addItem(pixmap_item)
                self.scene.setSceneRect(pixmap_item.boundingRect())
                self.graphicsView.setScene(self.scene)
                self.graphicsView.fitInView(self.scene.sceneRect(), Qt.KeepAspectRatio)
                self.comboBox.setCurrentIndex(self.c_index)
                self.pushButton_2.setEnabled(True)
                self.pushButton_3.setEnabled(True)
                self.pushButton_4.setEnabled(True)
                self.pushButton_5.setEnabled(True)
                self.pushButton_6.setEnabled(True)
            else:
                # print("This is the last one")
                QMessageBox.warning(self, "Warning", "This is the last one.")
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def Inverted_color(self):
        try:
            if self.image:
                img_v = self.pixmap_to_numpy(self.image)
                img_inverted = inverted_colors_func(img_v)
                img_inverted = self.numpy_to_pixmap(img_inverted)
                self.image=img_inverted
                pixmap_item = QGraphicsPixmapItem(img_inverted)
                self.scene.clear()
                self.scene.addItem(pixmap_item)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def pixmap_to_numpy(self,pixmap):
        try:
            image = QImage(pixmap.toImage())
            # print(image.format())  
            if image.format() in (QImage.Format_ARGB32, QImage.Format_RGB32, QImage.Format_ARGB32_Premultiplied):
                ptr = image.bits()
                ptr.setsize(image.byteCount())      
                arr = np.array(ptr).reshape(image.height(), image.width(), 4)   
                if (image.format() == QImage.Format_RGB32) or (image.format()== QImage.Format_ARGB32) :     
                    arr = arr[:, :, :3] 
                elif image.format()  == QImage.Format_ARGB32_Premultiplied:
                    # print(arr.shape)
                    # Handle premultiplied alpha
                    alpha = arr[:, :, 3:4] / 255.0
                    bgr = arr[:, :, :3] / alpha
                    # Clip values to ensure they are in valid range [0, 255]
                    bgr = np.clip(bgr, 0, 255)
                    # Convert to uint8 data type
                    bgr = bgr.astype(np.uint8)
                    # Convert from RGBA to BGR
                    arr = bgr[..., ::-1]
                    # Debugging: print the shape and dtype of the array
                # print(f"Array shape: {arr.shape}, dtype: {arr.dtype}")        
                return arr       
            # Handle QImage.Format_RGB888
            elif image.format()== QImage.Format_RGB888:
                width = image.width()
                height = image.height()
                ptr = image.bits()
                ptr.setsize(image.byteCount())               
                # Convert to numpy array
                arr = np.array(ptr).reshape(height, width, 3)               
                # Debugging: print the shape and dtype of the array
                # print(f"Array shape: {arr.shape}, dtype: {arr.dtype}")
                return arr
            else:
                raise ValueError("The image format does not support conversion to a numpy array.")
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def numpy_to_pixmap(self, numpy_array):
        try:
            if len(numpy_array.shape) == 2:  
                height, width = numpy_array.shape
                bytes_per_line = width
                q_image = QImage(numpy_array.data, width, height, bytes_per_line, QImage.Format_Grayscale8)
                return QPixmap.fromImage(q_image)
            elif len(numpy_array.shape) == 3:  
                height, width, channel = numpy_array.shape
                bytes_per_line = channel * width
                if channel == 3:
                    q_image = QImage(numpy_array.data, width, height, bytes_per_line, QImage.Format_RGB888)
                elif channel == 4:
                    q_image = QImage(numpy_array.data, width, height, bytes_per_line, QImage.Format_RGBA8888)
                else:
                    raise ValueError("Unsupported number of channels: {}".format(channel))
                return QPixmap.fromImage(q_image)
            else:
                raise ValueError("Unsupported image format: {}".format(numpy_array.shape))
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def show_mask(self, mask, random_color=False,i=0):
        try:
            if random_color:
                color = np.concatenate([np.random.random(3), np.array([0.6])], axis=0)
            else:
                color = np.array([30/255, 144/255, 255/255, 0.6])
            h, w = mask.shape[-2:]
            mask_image = mask.reshape(h, w, 1) * color.reshape(1, 1, -1)
            # cv2.imshow('mask'+str(i),mask_image)
            mask_image =  (mask_image * 255).astype(np.uint8)
            # cv2.imshow('mask',mask_image)
            # print(np.max(mask_image))
            img_mask = self.numpy_to_pixmap(mask_image)
            pixmap_item = QGraphicsPixmapItem(img_mask)
            self.scene.addItem(pixmap_item)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def SAM_segment(self):  # progress_signal
        try:
            # 检查 start_point 和 end_point 是否存在
            if not hasattr(self.graphicsView, 'start_point') or not hasattr(self.graphicsView, 'end_point'):
                QMessageBox.warning(self, "Warning", "Please select a rectangle area first.")
                return

            if self.graphicsView.start_point is None or self.graphicsView.end_point is None:
                QMessageBox.warning(self, "Warning", "Please select a rectangle area first.")
                return
            progress_dialog = QProgressDialog("Running now, please wait a moment...", "Cancel", 0, 100, self)
            progress_dialog.setWindowTitle("Processing")
            progress_dialog.setWindowModality(Qt.WindowModal)  
            progress_dialog.setAutoClose(True) 
            progress_dialog.setAutoReset(True)  
            progress_dialog.setMinimumDuration(0)  
            progress_dialog.setValue(0)  

            progress_dialog.setValue(10) 

            current_dir = os.path.dirname(os.path.abspath(__file__))
            checkpoint_path_1 = os.path.join(current_dir, "sam_vit_h_4b8939.pth")
            checkpoint_path_2 = os.path.join(current_dir, "sam_vit_b_01ec64.pth")
            if torch.cuda.is_available():
                if not os.path.exists(checkpoint_path_1):
                    QMessageBox.warning(self, "Warning",  "Please download the '.pth' file first.")
                    return
                sam = sam_model_registry['vit_h'](checkpoint=checkpoint_path_1)
                device = 'cuda'
            else:
                if not os.path.exists(checkpoint_path_2):
                    QMessageBox.warning(self, "Warning",  "Please download the '.pth' file first.")
                    return
                sam = sam_model_registry['vit_b'](checkpoint=checkpoint_path_2)
                device = 'cpu'

            sam.to(device=device)
            predictor = SamPredictor(sam)
            self.filename = self.image_files[self.c_index]
            file_path = os.path.join(self.folder_path, self.filename)

            if not os.path.exists(file_path):
                raise FileNotFoundError(f"File {file_path} doesn't exist")

            has_non_ascii = any(ord(c) > 127 for c in file_path)

            if has_non_ascii:
                temp_dir = tempfile.gettempdir()

                file_ext = os.path.splitext(file_path)[1]

                temp_filename = f"{uuid.uuid4().hex}{file_ext}"
                temp_path = os.path.join(temp_dir, temp_filename)

                try:
                    shutil.copy(file_path, temp_path)

                    image = cv2.imread(temp_path)

                    if image is None:
                        raise ValueError(f"Image cannot be read from temporary path: {temp_path}")

                finally:
                    if os.path.exists(temp_path):
                        os.remove(temp_path)
            else:

                image = cv2.imread(file_path)
                if image is None:
                    raise ValueError(f"Unable to read file: {file_path}")

            progress_dialog.setValue(30) 

            if image is not None:
                predictor.set_image(image)

                input_box = [self.graphicsView.start_point.x(), self.graphicsView.start_point.y(),
                            self.graphicsView.end_point.x(), self.graphicsView.end_point.y()]  #  (x_min, y_min, x_max, y_max)
                input_box = np.array(input_box)

                masks, scores, logits = predictor.predict(box=input_box)


                progress_dialog.setValue(70)  

                if masks.shape[0] != 0:
                    self.pushButton_7.setEnabled(False)
                    self.pushButton_8.setEnabled(False)
                    self.pushButton_11.setEnabled(False)
                    self.pushButton_12.setEnabled(True)
                    self.pushButton_13.setEnabled(True)
                    self.pushButton_15.setEnabled(True)
                    for i, (mask, score) in enumerate(zip(masks, scores)):
                        if i == 0:
                            self.show_mask(mask, True, i)
                            self.bin = (mask * 255).astype(np.uint8)
                            pixmap_result = self.numpy_to_pixmap(self.bin)
                            pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
                            self.scene_2.clear()
                            self.scene_3.clear()
                            self.scene_3.addItem(pixmap_result_item)
                            self.graphicsView_3.setScene(self.scene_3)
                            self.graphicsView_3.fitInView(self.graphicsView_3.sceneRect(), Qt.KeepAspectRatio)
                            self.graphicsView_4.setScene(self.scene_3)
                            self.graphicsView_4.fitInView(self.graphicsView_4.sceneRect(), Qt.KeepAspectRatio)
                            progress_dialog.setValue(100)  
                            QMessageBox.information(None, "Success", "You can also try using the 'Polygon Tool'.") 
                              
                            
            else:
                QMessageBox.warning(self, "Warning", "Failed to read image. Please check the file path.")
        except ValueError as e:
            QMessageBox.warning(self, 'Error', f'{e}')

    
    def wheelEvent(self, event):
        try:
            pos=event.pos()
            pos_adjusted = QtCore.QPoint(pos.x() - 10, pos.y() - 60)  
            if self.graphicsView.viewport().rect().contains(pos_adjusted):
                if(event.angleDelta().y()>0.5):
                    self.graphicsView.scale(1.5,1.5)
                elif(event.angleDelta().y()< 0.5):
                    self.graphicsView.scale(1 /1.5,1 / 1.5)

            pos_adjusted_2 = QtCore.QPoint(pos.x() - 610, pos.y() - 60)  
            if self.graphicsView_2.viewport().rect().contains(pos_adjusted_2):
                if(event.angleDelta().y()>0.5):
                    self.graphicsView_2.scale(1.5,1.5)
                elif(event.angleDelta().y()< 0.5):
                    self.graphicsView_2.scale(1 /1.5,1 / 1.5)

            pos_adjusted_3 = QtCore.QPoint(pos.x() - 610, pos.y() - 390) 
            if self.graphicsView_3.viewport().rect().contains(pos_adjusted_3):
                if(event.angleDelta().y()>0.5):
                    self.graphicsView_3.scale(1.5,1.5)
                elif(event.angleDelta().y()< 0.5):
                    self.graphicsView_3.scale(1 /1.5,1 / 1.5)

            pos_adjusted_4 = QtCore.QPoint(pos.x() - 1100, pos.y() - 60) 
            if self.graphicsView_4.viewport().rect().contains(pos_adjusted_4):
                if(event.angleDelta().y()>0.5):
                    self.graphicsView_4.scale(1.5,1.5)
                elif(event.angleDelta().y()< 0.5):
                    self.graphicsView_4.scale(1 /1.5,1 / 1.5)    
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return


    def grayscale(self):
        try:
            self.gray_img = grayscale_func(self.crop_result)
            pixmap_result = self.numpy_to_pixmap(self.gray_img)
            pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
            self.scene_2.clear()
            self.scene_2.addItem(pixmap_result_item)
            self.graphicsView_2.setScene(self.scene_2)
            self.graphicsView_2.fitInView(self.graphicsView_2.sceneRect(), Qt.KeepAspectRatio)
            self.pushButton_7.setEnabled(True)
            self.pushButton_11.setEnabled(True)
            self.radioButton_3.setEnabled(True)
            self.radioButton_4.setEnabled(True)

        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return


    def enhancement(self):
        try:
            self.gray_img=enhancement_func(self.gray_img)
            pixmap_result = self.numpy_to_pixmap(self.gray_img)
            pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
            self.scene_2.clear()
            self.scene_2.addItem(pixmap_result_item)
            self.graphicsView_2.setScene(self.scene_2)
            self.graphicsView_2.fitInView(self.graphicsView_2.sceneRect(), Qt.KeepAspectRatio)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def units_set(self):
        try:
            selectedButton =self.buttonGroup.checkedButton()
            id=selectedButton.text()
            # print(id)
            if (id=='mm'):
                self.unit=1
            elif (id=='inch'):
                self.unit=2
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def method_set(self):
        try:
            selectedButton =self.buttonGroup_2.checkedButton()
            id=selectedButton.text()
            # print(id)
            if (id=='Image Binarization'):
                self.pushButton_11.setEnabled(True)
                self.pushButton_12.setEnabled(True)
                self.pushButton_13.setEnabled(True)
                self.pushButton_14.setEnabled(False)
            elif (id=='Edge Detection'):
                self.pushButton_11.setEnabled(False)
                self.pushButton_12.setEnabled(False)
                self.pushButton_13.setEnabled(False)
                self.pushButton_14.setEnabled(True)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def binarization(self):
        try:
            self.bin = binarize_func(self.gray_img)
            self.bin = cv2.bitwise_not(self.bin)
            pixmap_result = self.numpy_to_pixmap(self.bin)
            pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
            self.scene_3.clear()
            self.scene_3.addItem(pixmap_result_item)
            self.graphicsView_3.setScene(self.scene_3)
            self.graphicsView_3.fitInView(self.graphicsView_3.sceneRect(), Qt.KeepAspectRatio)
            self.graphicsView_4.setScene(self.scene_3)
            self.graphicsView_4.fitInView(self.graphicsView_4.sceneRect(), Qt.KeepAspectRatio)
            self.pushButton_12.setEnabled(True)
            self.pushButton_13.setEnabled(True)
            # self.pushButton_14.setEnabled(True)
            self.pushButton_15.setEnabled(True)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def corrosion(self):
        try:
            circle=self.textEdit_6.toPlainText()
            circle=int(circle)
            self.bin=corrosion_func(self.bin,circle)
            pixmap_result = self.numpy_to_pixmap(self.bin)
            pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
            self.scene_3.clear()
            self.scene_3.addItem(pixmap_result_item)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return


    def dilation(self):
        try:
            self.bin = dilation_func(self.bin)
            pixmap_result = self.numpy_to_pixmap(self.bin)
            pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
            self.scene_3.clear()
            self.scene_3.addItem(pixmap_result_item)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def chain_code(self):
        try:
            scene_rect = self.scene_3.sceneRect()
            pixmap = QPixmap(scene_rect.size().toSize())
            pixmap.fill(Qt.transparent)
            painter = QPainter(pixmap)
            self.scene_3.render(painter, QRectF(pixmap.rect()), scene_rect)
            painter.end()
            image = pixmap.toImage()
            image.save("scene_image.png")
            # print("Image saved")
            
            self.bin = cv2.imread("scene_image.png",cv2.IMREAD_GRAYSCALE)
            # self.bin = self.bin[:,:,0]
            contours, _ = cv2.findContours(self.bin, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
            
            if contours:
                max_contour = max(contours, key=cv2.contourArea)
                max_contour=np.reshape(max_contour, (max_contour.shape[0], max_contour.shape[2]))        
                # print(max_contour[0][0])
                result_image = np.zeros_like(self.bin)
                # print(self.bin.shape)
                cv2.drawContours(result_image, [max_contour], -1, 255, thickness=1)
                cv2.imwrite('result_image.png',result_image)
                max_contour[:, [0, 1]] = max_contour[:, [1, 0]]
                self.boundary=max_contour
                self.chaincode,oringin = gui_chain_code_func(result_image,max_contour[0])
                # print(self.chaincode.shape)

                # Check if chain_code is not empty
                if len(self.chaincode) == 0:
                    QMessageBox.warning(self, "Warning", "Chain code is empty")
                    return

                # Draw green line for boundary
                pen = QPen(Qt.green, 3)
                # print(self.boundary)
                for i in range(len(self.boundary)-1):
                    start_point = QPointF(self.boundary[i][1], self.boundary[i][0])
                    end_point = QPointF(self.boundary[i + 1][1], self.boundary[i + 1][0])
                    line_item_1 = QGraphicsLineItem(start_point.x(), start_point.y(), end_point.x(), end_point.y())
                    line_item_1.setPen(pen)
                    self.scene_3.addItem(line_item_1)

                x_ = calc_traversal_dist(self.chaincode)
                x = np.vstack(([0, 0], x_))
                # print(x)

                # # Draw red line for chain code traversal
                pen.setColor(Qt.red)
                pen.setWidth(2)
                origin = self.boundary[0]
                current_point = QPointF(origin[1],origin[0])
                for move in x:
                    next_point = QPointF(origin[1] + move[0],origin[0]  - move[1])
                    # print(next_point)
                    line_item = QGraphicsLineItem(current_point.x(), current_point.y(), next_point.x(), next_point.y())
                    line_item.setPen(pen)
                    self.scene_3.addItem(line_item)
                    current_point = next_point                    
                is_closed,endpoint = is_completed_chain_code(self.chaincode, self.boundary[0])
                self.graphicsView_4.fitInView(self.graphicsView_4.sceneRect(), Qt.KeepAspectRatio)
                # Get the current transform
                transform = self.graphicsView_4.transform()
                # Apply the scale transformation
                transform.scale(7, 7)
                self.graphicsView_4.setTransform(transform)
                # Center the view on the specified point
                self.graphicsView_4.centerOn(QPointF(endpoint[1],endpoint[0]))

                if not is_closed:
                    QMessageBox.critical(None, "Error", f"Chain code is not closed (length is {self.chaincode.shape[0]}), please edit.")

                else:
                    QMessageBox.information(None, "Success", f"Chain code is closed, and length is {self.chaincode.shape[0]}")               
                    # print(self.chaincode)
                    # print(self.chaincode.shape)
                    self.pushButton_9.setEnabled(True)
                    self.pushButton_10.setEnabled(True)
                    
            else:
                # print("not found boundary")
                QMessageBox.warning(self, "Warning", "not found boundary")
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def functions_selection(self):
        try:
            selected_item = self.comboBox_2.currentText()
            min_value = self.horizontalSlider.minimum()
            max_value = self.horizontalSlider.maximum()
            current_value = self.horizontalSlider.value()
            threshvalue = 255*(current_value-min_value)/(max_value-min_value)
            if selected_item=="canny":
                self.bin=contour_extraction_func(self.gray_img,1,threshvalue/2,threshvalue)
            elif selected_item=="sobel":
                self.bin=contour_extraction_func(self.gray_img,2,threshvalue/2,threshvalue)
            elif selected_item=="zerocross":
                self.bin=contour_extraction_func(self.gray_img,3,threshvalue/2,threshvalue)   
            elif selected_item=="laplace":
                self.bin=contour_extraction_func(self.gray_img,4,threshvalue/2,threshvalue)    
            elif selected_item=="Roberts":
                self.bin=contour_extraction_func(self.gray_img,5,threshvalue/2,threshvalue)  
            elif selected_item=="Prewitt":
                self.bin=contour_extraction_func(self.gray_img,6,threshvalue/2,threshvalue) 
            else:
                QMessageBox.warning(self, 'Error', 'Please select the an operator') 
                return

            pixmap_result = self.numpy_to_pixmap(self.bin)
            pixmap_result_item = QGraphicsPixmapItem(pixmap_result)
            self.scene_3.clear()
            self.scene_3.addItem(pixmap_result_item) 
            self.graphicsView_3.setScene(self.scene_3)
            self.graphicsView_3.fitInView(self.graphicsView_3.sceneRect(), Qt.KeepAspectRatio)
            self.graphicsView_4.setScene(self.scene_3)
            self.graphicsView_4.fitInView(self.graphicsView_4.sceneRect(), Qt.KeepAspectRatio) 
            self.pushButton_15.setEnabled(True)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def clicked_point(self):
        self.flag_point=1
        self.pushButton_16.setEnabled(True)

    def save(self):
        if self.textEdit_7.toPlainText():
            id_full = self.textEdit_7.toPlainText()
            self.id_full = id_full
            self.filename = self.image_files[self.c_index]

            if not os.path.exists('results'):
                os.makedirs('results')
            if not os.path.exists('label'):
                os.makedirs('label')
            # Writing boundary data to file
            boundary_filename = f"results/{self.filename[:-4]}_{id_full}_b.txt"

            write_data_to_file(self.boundary, boundary_filename)

            # Writing chain code data to file
            chain_filename = f"results/{self.filename[:-4]}_{id_full}_c.txt"
            write_data_to_file(self.chaincode, chain_filename)

            # Writing image to file
            image_filename = f"label/{self.filename[:-4]}_{id_full}.png"
            cv2.imwrite(image_filename,self.bin)
            
            try:
                # Processing distance data
                dis = float(self.textEdit_5.toPlainText())
                if dis>0:
                    dis_pixel = self.distance / dis
                    dis_mm = dis / self.distance
                else:
                    dis_pixel=0
                    dis_mm=0
                [area,circumference]=cal_area_c(self.chaincode,self.boundary)


                # Writing results to Excel file
                if self.unit==1:
                    results = {
                        'filepath': [f"{self.filename[:-4]}_{id_full}"],
                        'scale:pixels/mm': [dis_pixel],
                        'scale:mm/pixel': [dis_mm],
                        'circumference:pixel': [circumference],  
                        'area:pixel': [area],  # Example calculation
                        'circumference:mm': [circumference * dis_mm],  # Example calculation
                        'area:mm^2': [area * dis_mm ** 2],  # Example calculation
                    }
                elif self.unit==2:
                    results = {
                        'filepath': [f"{self.filename[:-4]}_{id_full}"],
                        'scale:pixels/inch': [dis_pixel],
                        'scale:inch/pixel': [dis_mm],
                        'circumference:pixel': [circumference],  
                        'area:pixel': [area],  # Example calculation
                        'circumference:inch': [circumference * dis_mm],  # Example calculation
                        'area:inch^2': [area * dis_mm ** 2],  # Example calculation
                    }
                df = pd.DataFrame(results)
                df.to_excel(f"results/{self.filename[:-4]}_{id_full}_info.xlsx", index=False,sheet_name='Sheet1')
                QMessageBox.information(None, "Success",'    done!   ')
                self.pushButton_17.setEnabled(True)
            except Exception as e:
                QMessageBox.critical(None, "Error", str(e))
        else:
            # print('Chain code is null. /Chain code is not closed.')
            QMessageBox.warning(self, 'Error', 'Chain code is null. /Chain code is not closed.')      
        


    def go_to_window2(self):
        # QApplication.setAttribute(Qt.AA_EnableHighDpiScaling)
        try:
            self.window2 = MainWindow_1(self.chaincode,self.filename,self.id_full)
            self.window2.show()
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

    def skip(self):
        try:
            self.graphicsView.points.append((0,0))
            self.graphicsView.points.append((0,0))

            # print(self.points)

            self.distance = 0
            self.textEdit_5.setText(str(0))
            self.pushButton_16.setEnabled(True)
        except ValueError as e:
            QMessageBox.warning(self,'Error','{e}')
            return

def main():
    QApplication.setAttribute(Qt.AA_EnableHighDpiScaling)
    app = QApplication(sys.argv)
    Main_Window = MainWindow()
    Main_Window.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'EFA.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.
import os
os.environ["KMP_DUPLICATE_LIB_OK"] = "TRUE"
import numpy as np
import pandas as pd

from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtWidgets import *
from PyQt5.QtGui import QImage, QPixmap, QPainterPath, QPainter,QPen,QColor
from PyQt5.QtCore import Qt
import sys
# import openpyxl


sys.path.append('/functions')
from functions.fourier_approx_norm_modify import fourier_approx_norm_modify
from functions.calc_dc_components_modify import calc_dc_components_modify
from functions.code2axis import code2axis


class MainWindow_1(QMainWindow):
    def __init__(self,chaincode,filename,id_full):
        super().__init__()
        self.setObjectName("MainWindow")
        self.resize(1143, 543)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.setFont(font)
        self.centralwidget = QtWidgets.QWidget(self)
        self.centralwidget.setObjectName("centralwidget")
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox.setGeometry(QtCore.QRect(20, 80, 221, 401))
        self.groupBox.setObjectName("groupBox")
        
        self.checkBox = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBox.setGeometry(QtCore.QRect(70, 100, 141, 51))
        self.checkBox.setObjectName("checkBox")
        self.checkBox.setChecked(True)
        self.checkBox_2 = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBox_2.setGeometry(QtCore.QRect(70, 150, 141, 51))
        self.checkBox_2.setObjectName("checkBox_2")
        self.checkBox_2.setChecked(True)
        self.checkBox_3 = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBox_3.setGeometry(QtCore.QRect(70, 200, 141, 51))
        self.checkBox_3.setObjectName("checkBox_3")
        self.checkBox_3.setChecked(True)
        self.checkBox_4 = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBox_4.setGeometry(QtCore.QRect(70, 250, 141, 51))
        self.checkBox_4.setObjectName("checkBox_4")
        self.checkBox_4.setChecked(True)
        self.checkBox_5 = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBox_5.setGeometry(QtCore.QRect(70, 300, 141, 51))
        self.checkBox_5.setObjectName("checkBox_5")
        self.checkBox_5.setChecked(True)
        self.checkBox_6 = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBox_6.setGeometry(QtCore.QRect(70, 360, 141, 51))
        self.checkBox_6.setObjectName("checkBox_6")
        self.checkBox_6.setChecked(True)
        self.checkBox_7 = QtWidgets.QCheckBox(self.centralwidget)
        self.checkBox_7.setGeometry(QtCore.QRect(70, 420, 141, 51))
        self.checkBox_7.setObjectName("checkBox_7")
        self.checkBox_7.setChecked(True)

        self.widget = QtWidgets.QWidget(self.centralwidget)
        self.widget.setGeometry(QtCore.QRect(250, 90, 421, 371))
        self.widget.setObjectName("widget")
        self.widget_3 = QtWidgets.QWidget(self.centralwidget)
        self.widget_3.setGeometry(QtCore.QRect(690, 90, 431, 371))
        self.widget_3.setObjectName("widget_3")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setGeometry(QtCore.QRect(310, 50, 221, 21))
        self.label.setObjectName("label")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setGeometry(QtCore.QRect(310, 10, 231, 21))
        self.label_2.setObjectName("label_2")
        self.textEdit = QtWidgets.QTextEdit(self.centralwidget)
        self.textEdit.setGeometry(QtCore.QRect(550, 10, 81, 31))
        self.textEdit.setObjectName("textEdit")
        self.textEdit_2 = QtWidgets.QTextEdit(self.centralwidget)
        self.textEdit_2.setGeometry(QtCore.QRect(550, 50, 81, 31))
        self.textEdit_2.setObjectName("textEdit_2")
        self.pushButton = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton.setGeometry(QtCore.QRect(700, 10, 201, 31))
        self.pushButton.setObjectName("pushButton")
        self.pushButton_2 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_2.setGeometry(QtCore.QRect(700, 50, 201, 31))
        self.pushButton_2.setObjectName("pushButton_2")
        self.pushButton_3 = QtWidgets.QPushButton(self.centralwidget)
        self.pushButton_3.setGeometry(QtCore.QRect(910, 50, 201, 31))
        self.pushButton_3.setObjectName("pushButton_3")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setGeometry(QtCore.QRect(250, 470, 421, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setGeometry(QtCore.QRect(750, 470, 421, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.graphicsView = QtWidgets.QGraphicsView(self.centralwidget)
        self.graphicsView.setGeometry(QtCore.QRect(250, 90, 421, 371))
        self.graphicsView.setObjectName("graphicsView")
        self.graphicsView_2 = QtWidgets.QGraphicsView(self.centralwidget)
        self.graphicsView_2.setGeometry(QtCore.QRect(690, 90, 431, 371))
        self.graphicsView_2.setObjectName("graphicsView_2")
        self.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(self)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1143, 25))
        self.menubar.setObjectName("menubar")
        self.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(self)
        self.statusbar.setObjectName("statusbar")
        self.setStatusBar(self.statusbar)

        _translate = QtCore.QCoreApplication.translate
        self.setWindowTitle(_translate("MainWindow", "EFA"))
        self.checkBox.setText(_translate("MainWindow", "Translatoin"))
        self.checkBox_2.setText(_translate("MainWindow", "Reverse"))
        self.checkBox_3.setText(_translate("MainWindow", "Scale"))
        self.checkBox_4.setText(_translate("MainWindow", "Rotate"))
        self.checkBox_5.setText(_translate("MainWindow", "Starting point"))
        self.checkBox_6.setText(_translate("MainWindow", "Y-symmetric"))
        self.checkBox_7.setText(_translate("MainWindow", "X-symmetric"))
        self.groupBox.setTitle(_translate("MainWindow", "normalization options"))
        self.label.setText(_translate("MainWindow", "Visualized harmonic number:"))
        self.label_2.setText(_translate("MainWindow", "Maximum harmonic number:"))
        self.pushButton.setText(_translate("MainWindow", "Calculate and save "))
        self.pushButton_2.setText(_translate("MainWindow", "Plot curve"))
        self.pushButton_3.setText(_translate("MainWindow", "Save curve"))
        self.pushButton_2.setEnabled(False)
        self.pushButton_3.setEnabled(False)
        self.textEdit.setText("35")
        self.textEdit_2.setText("35")
        self.label_3.setText(_translate("self", "reconstructed shapes using EFDs (red line) and chain-code (bhue line)"))
        self.label_4.setText(_translate("self", "reconstructed shapes using normalized EFDs"))

        self.scene = QGraphicsScene()
        self.scene_1 = QGraphicsScene()
        self.chaincode = chaincode
        self.id_full = id_full
        self.filename = filename
        self.option = [1,1,1,1,1,1,1]

        self.checkBox_4.stateChanged.connect(self.onCheckBox4StateChanged)
        self.checkBox_5.stateChanged.connect(self.onCheckBox4StateChanged)
        self.pushButton.clicked.connect(self.cal_hs)
        self.pushButton_2.clicked.connect(self.plot_hs)
        self.pushButton_3.clicked.connect(self.save_hs)


    def onCheckBox4StateChanged(self):
        # Assuming you have imported Tkinter as tk
        # and you have a Tkinter variable handles
        t_1 = self.checkBox_4.isChecked()
        t_2 = self.checkBox_5.isChecked()

        if t_1 == 1 and t_2 == 1:
            self.checkBox_6.setEnabled(True)
            self.checkBox_7.setEnabled(True)
        else:
            self.checkBox_6.setChecked(False) 
            self.checkBox_7.setChecked(False) 
            self.checkBox_6.setEnabled(False)
            self.checkBox_7.setEnabled(False)


    def cal_hs(self):
        circle = self.textEdit.toPlainText()
        if circle.isdigit():
            numofharmoinc = int(circle)
            if numofharmoinc <= 0:
                QMessageBox.warning(self, '', 'please enter the positive number.')
                return
        else:
            QMessageBox.warning(self, '', 'please enter the number.')
            return
        
        self.option = [
            self.checkBox_4.isChecked(),
            self.checkBox_3.isChecked(),
            self.checkBox_2.isChecked(),
            self.checkBox_6.isChecked(),
            self.checkBox_7.isChecked(),
            self.checkBox_5.isChecked(),
            self.checkBox.isChecked()
        ]
        # 假设 chaincode 是一个 (N, 2) 的数组，表示 2D 点
        # num_points = 1000  # 降采样后的点数
        # indices = np.linspace(0, len(self.chaincode) - 1, num_points).astype(int)
        # simplified_chaincode = self.chaincode[indices]

        _, a, b, c, d = fourier_approx_norm_modify(self.chaincode, numofharmoinc, 400, 1,  self.option)
        t=np.transpose([a ,b, c, d])
        Hs = np.reshape(t, (1, -1))
        coffs = [["filepath"]]
        cols = numofharmoinc * 4
        # matrix = []
        # for col in range(1, cols + 1):
        #     letter = chr(ord('a') + (col - 1) % 4)
        #     number = str((col - 1) // 4 + 1)
        #     matrix.append(letter + number)
        matrix = [f"{chr(97 + (col - 1) % 4)}{(col - 1) // 4 + 1}" for col in range(1, cols + 1)]
        coffs[0].extend(matrix)
        coffs.append([f"{self.filename[:-4]}_{self.id_full}"])
        coffs[1].extend(Hs.flatten().tolist())

        df = pd.DataFrame(coffs)
        # 确保目录存在
        output_dir = "results/"
        os.makedirs(output_dir, exist_ok=True)
        with pd.ExcelWriter(f"results/{self.filename[:-4]}_{self.id_full}_info.xlsx", engine='openpyxl', mode='a', if_sheet_exists='replace') as writer:
            df.to_excel(writer, sheet_name='Sheet2', index=False, header=False)
        QMessageBox.information(None, "Success", "  done!  ")
        self.pushButton_2.setEnabled(True)
        self.pushButton_3.setEnabled(True)   


    def plot_hs(self):
        self.scene.clear()
        self.scene_1.clear()
        chain_code = self.chaincode
        if (self.textEdit.toPlainText()).isdigit():
            max_numofharmoinc = int(self.textEdit.toPlainText())
            if max_numofharmoinc <= 0:
                QMessageBox.warning(self, '', 'please enter the positive number.')
                return
        else:
            QMessageBox.warning(self, '', 'please enter the number.')
            return


        if chain_code.size > 0:
            contour_points = np.array([0, 0])
            chain_points = code2axis(chain_code, contour_points)
            for i in range(len(chain_points)-1):
                x1, y1 = chain_points[i,:]
                x2, y2 = chain_points[i + 1,:]
                line = QGraphicsLineItem(x1, y1, x2, y2)
                line.setPen(QPen(QColor(0, 0, 255)))
                self.scene.addItem(line)
                self.scene_1.setSceneRect(self.scene_1.itemsBoundingRect()) 

            circle = self.textEdit_2.toPlainText()
            if circle.isdigit():
                numofharmoinc = int(circle)
                if numofharmoinc <= 0:
                    QMessageBox.warning(self, '', 'please enter the positive number.')
                    return
                if numofharmoinc > max_numofharmoinc:
                    QMessageBox.warning(self, 'message', f"The input must be less than {max_numofharmoinc}")
                else:
                    ro = self.checkBox_4.isChecked()
                    sc = self.checkBox_3.isChecked()
                    re = self.checkBox_2.isChecked()
                    y_sy = self.checkBox_6.isChecked()
                    x_sy = self.checkBox_7.isChecked()
                    sta = self.checkBox_5.isChecked()
                    trans = self.checkBox.isChecked()
                    option = [ro, sc, re, y_sy, x_sy, sta, trans]

                    x_,_,_,_,_ = fourier_approx_norm_modify(chain_code, numofharmoinc, 400, 0, option)
                    chain_points_approx = np.vstack((x_, x_[0,:]))
                    # print(chain_points_approx)

                    for i in range(len(chain_points_approx)-1):
                        x1  = chain_points_approx[i,0] + contour_points[0]
                        y1 = contour_points[1] - chain_points_approx[i, 1]

                        x2  = chain_points_approx[i+1,0] + contour_points[0]
                        y2 = contour_points[1] - chain_points_approx[i+1, 1]

                        line = QGraphicsLineItem(y1, x1, y2, x2)
                        line.setPen(QPen(QColor(255, 0, 0)))
                        self.scene.addItem(line)
                    self.graphicsView.setScene(self.scene)
                    self.graphicsView.fitInView(self.graphicsView.sceneRect(), Qt.KeepAspectRatio)

                    
                    x_,_,_,_,_ = fourier_approx_norm_modify(chain_code, numofharmoinc, 400, 1, option)
                    chain_points_approx = x_
                    for i in range(len(chain_points_approx)-1):
                        x1  = chain_points_approx[i,0] + contour_points[0]
                        y1 = contour_points[1] - chain_points_approx[i, 1]

                        x2  = chain_points_approx[i+1,0] + contour_points[0]
                        y2 = contour_points[1] - chain_points_approx[i+1, 1]

                        line = QGraphicsLineItem(y1, x1, y2, x2)
                        line.setPen(QPen(QColor(255, 0, 0),0.01))
                        
                        self.scene_1.addItem(line)
                        self.scene_1.setSceneRect(self.scene_1.itemsBoundingRect()) 

                    self.graphicsView_2.setScene(self.scene_1)
                    self.graphicsView_2.fitInView(self.graphicsView_2.sceneRect(), Qt.KeepAspectRatio)
            else:
                QMessageBox.warning(self, '', 'please enter the number.')
                return
                
        else:
            print('Chain code is null')
            QMessageBox.warning(self, '', 'Chain code is null')





    # def plot_hs(self):
    #     self.scene.clear()
    #     self.scene_1.clear()
    #     chain_code = self.chaincode
    #     # a = self.a
    #     # b = self.b 
    #     # c = self.c
    #     # d = self.d
    #     max_numofharmoinc = int(self.textEdit.toPlainText())

    #     if chain_code.size > 0:
    #         contour_points = np.array([0, 0])
    #         chain_points = code2axis(chain_code, contour_points)
    #         for i in range(len(chain_points)-1):
    #             x1, y1 = chain_points[i,:]
    #             x2, y2 = chain_points[i + 1,:]
    #             line = QGraphicsLineItem(x1, y1, x2, y2)
    #             line.setPen(QPen(QColor(0, 0, 255)))
    #             self.scene.addItem(line)
    #             self.scene_1.setSceneRect(self.scene_1.itemsBoundingRect()) 

    #         circle = self.textEdit_2.toPlainText()
    #         numofharmoinc = int(circle)
    #         if numofharmoinc > max_numofharmoinc:
    #             QMessageBox.warning(self, 'message', f"The input must be less than {max_numofharmoinc}")
    #         else:

    #             x_, _, _, _, _ = fourier_approx_norm_modify(chain_code, numofharmoinc, 400, 0, self.option)
    #             # x_2, _, _, _, _  = fourier_approx_norm_modify(chain_code, numofharmoinc, 400, 1, self.option)


    #             chain_points_approx_1 = np.vstack((x_, x_[0,:]))
    #             # print(chain_points_approx)
    #             x_coords_1 = chain_points_approx_1[:, 0] + contour_points[0]
    #             y_coords_1 = contour_points[1] - chain_points_approx_1[:, 1]

    #             # pen_red = QPen(QColor(255, 0, 0))
    #             for x1, y1, x2, y2 in zip(x_coords_1[:-1], y_coords_1[:-1], x_coords_1[1:], y_coords_1[1:]):
    #             # for i in range(len(chain_points_approx)-1):
    #             #     x1  = chain_points_approx[i,0] + contour_points[0]
    #             #     y1 = contour_points[1] - chain_points_approx[i, 1]

    #             #     x2  = chain_points_approx[i+1,0] + contour_points[0]
    #             #     y2 = contour_points[1] - chain_points_approx[i+1, 1]

    #                 line = QGraphicsLineItem(y1, x1, y2, x2)
    #                 line.setPen(QPen(QColor(255, 0, 0)))
    #                 self.scene.addItem(line)
    #             self.graphicsView.setScene(self.scene)
    #             self.graphicsView.fitInView(self.graphicsView.sceneRect(), Qt.KeepAspectRatio)

                
    #             x_2,_,_,_,_ = fourier_approx_norm_modify(chain_code, numofharmoinc, 400, 1, mode, option)
    #             chain_points_approx = x_2
    #             # _, _, Tk, T = calc_dc_components_modify(chain_code, 0)
    #             # output = np.zeros((400, 2))
    #             # for t in range(400):
    #             #     x_2 = 0.0
    #             #     y_2 = 0.0
    #             #     for i in range(numofharmoinc):
    #             #         x_2 += (a[i] * np.cos(2 * (i + 1) * np.pi * (t) * Tk / (400 - 1) / T) + b[i] * np.sin(2 * (i + 1) * np.pi * (t) * Tk / (400 - 1) / T))
    #             #         y_2 += (c[i] * np.cos(2 * (i + 1) * np.pi * (t) * Tk / (400 - 1) / T) + d[i] * np.sin(2 * (i + 1) * np.pi * (t) * Tk / (400 - 1) / T))
    #             #     output[t, 0] =  x_2
    #             #     output[t, 1] =  y_2
    #             # chain_points_approx = output
    #             for i in range(len(chain_points_approx)-1):
    #                 x1  = chain_points_approx[i,0] + contour_points[0]
    #                 y1 = contour_points[1] - chain_points_approx[i, 1]

    #                 x2  = chain_points_approx[i+1,0] + contour_points[0]
    #                 y2 = contour_points[1] - chain_points_approx[i+1, 1]

    #                 line = QGraphicsLineItem(y1, x1, y2, x2)
    #                 line.setPen(QPen(QColor(255, 0, 0),0.01))
                    
    #                 self.scene_1.addItem(line)
    #                 self.scene_1.setSceneRect(self.scene_1.itemsBoundingRect()) 

    #             self.graphicsView_2.setScene(self.scene_1)
    #             self.graphicsView_2.fitInView(self.graphicsView_2.sceneRect(), Qt.KeepAspectRatio)

    #     else:
    #         print('Chain code is null')
    #         QMessageBox.warning(self, '', 'Chain code is null')

    def save_hs(self):
        options = QFileDialog.Options()
        defaultPath = './results/'
        defaultFileName = f"{defaultPath}{'_'.join(self.filename.split('.')[:-1])}_{self.id_full}.png"
        filePath, _ = QFileDialog.getSaveFileName(self, "Save File", defaultFileName, 
                                                  "PNG files (*.png);;JPEG files (*.jpg);;All Files (*)", options=options)

        if filePath:
            extension = filePath.split('.')[-1].lower()
            pixmap = QApplication.primaryScreen().grabWindow(self.winId()) 
            if extension == 'png':                
                pixmap.save(filePath+'.png')
            elif extension == 'jpg':
                pixmap.save(filePath+'.jpg')
            else:
                print('Unsupported file format. Image not saved.')

            print(f'Image saved as: {filePath}')





import numpy as np
from calc_traversal_time import calc_traversal_time
from calc_traversal_dist import calc_traversal_dist


def calc_dc_components_modify(ai, mode):
    k = ai.shape[0]
    t = calc_traversal_time(ai)
    d = calc_traversal_dist(ai)
    Tk = t[k-1]
    
    if mode == 0:
        edist = d[k-1, 0]**2 + d[k-1, 1]**2
        if edist > 2:
            print('error chaincode, not close form')
            return None, None, None, None
        else:
            if edist > 0:
                vect = [-d[k-1, 0], -d[k-1, 1]]
                if vect[0] == 1 and vect[1] == 0:
                    ai = np.append(ai, 0)
                elif vect[0] == 1 and vect[1] == 1:
                    ai = np.append(ai, 1)
                elif vect[0] == 0 and vect[1] == 1:
                    ai = np.append(ai, 2)
                elif vect[0] == -1 and vect[1] == 1:
                    ai = np.append(ai, 3)
                elif vect[0] == -1 and vect[1] == 0:
                    ai = np.append(ai, 4)
                elif vect[0] == -1 and vect[1] == -1:
                    ai = np.append(ai, 5)
                elif vect[0] == 0 and vect[1] == -1:
                    ai = np.append(ai, 6)
                elif vect[0] == 1 and vect[1] == -1:
                    ai = np.append(ai, 7)

    elif mode == 1:
        if d[k-1, 0] != 0:
            exp1 = np.zeros(abs(d[k-1, 0])) + 2 * np.sign(d[k-1, 0]) + 2 * np.sign(d[k-1, 0]) * np.sign(d[k-1, 0])
            ai = np.append(ai, exp1)
        if d[-1, 1] != 0:
            exp2 = np.zeros(abs(d[k-1, 1])) + 2 + 2 * np.sign(d[k-1, 1]) + 2 * np.sign(d[k-1, 1]) * np.sign(d[k-1, 1])
            ai = np.append(ai, exp2)

    elif mode == 2:
        if d[k-1, 0] != 0 or d[k-1, 1] != 0:
            exp = (ai + 4) % 8
            ai = np.append(ai, exp)
    k = ai.shape[0]
    t = calc_traversal_time(ai)
    d = calc_traversal_dist(ai)
    T = t[k-1]
    
    sum_a0 = 0
    sum_c0 = 0
    for p in range(1, k):
        if p >=1:
            dp_prev = d[p - 1,:]
        else:
            dp_prev = np.zeros(2)
        delta_t = calc_traversal_time(ai[p])
        sum_a0 += (d[p, 0] + dp_prev[0]) * delta_t / 2
        sum_c0 += (d[p, 1] + dp_prev[1]) * delta_t / 2
    
    A0 = sum_a0 / T
    C0 = sum_c0 / T
    
    return A0, C0, Tk, T



import cv2
import numpy as np

def dilation_func(imBina):
    # Define the structuring element
    B = np.array([[0, 1, 0],
                  [1, 1, 1],
                  [0, 1, 0]], dtype=np.uint8)
    
    # Perform dilation
    im_dil = cv2.dilate(imBina.astype(np.uint8), B)
    
    return im_dil



from concurrent.futures import ProcessPoolExecutor
import numpy as np
import sys
import os

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

from calc_dc_components_modify import calc_dc_components_modify
from calc_harmonic_coefficients_modify import calc_harmonic_coefficients_modify


def compute_harmonic_coefficients(ai, harmonic_index):
    """单独计算一个谐波的系数"""
    return calc_harmonic_coefficients_modify(ai, harmonic_index + 1, 0)


def fourier_approx_norm_modify(ai, n, m, normalized, option):
    EPS = 1e-10
    a = np.zeros(n)
    b = np.zeros(n)
    c = np.zeros(n)
    d = np.zeros(n)

    # 并行计算谐波系数
    with ProcessPoolExecutor() as executor:
        results = list(executor.map(compute_harmonic_coefficients, [ai] * n, range(n)))
    
    # 收集计算结果
    for i, harmonic_coeff in enumerate(results):
        a[i] = harmonic_coeff[0]
        b[i] = harmonic_coeff[1]
        c[i] = harmonic_coeff[2]
        d[i] = harmonic_coeff[3]

    # 原有代码继续执行
    A0, C0, Tk, T = calc_dc_components_modify(ai, 0)

    # Normalization procedure
    if normalized:
        ro = option[0]
        sc = option[1]
        re = option[2]
        y_sy = option[3]
        x_sy = option[4]
        sta = option[5]
        trans = option[6]

        # Remove DC components
        if trans:
            A0 = 0
            C0 = 0

        if re:
            CrossProduct = a[0] * d[0] - c[0] * b[0]
            if CrossProduct < 0:
                b = -b
                d = -d

        tan_theta2 = 2 * (a[0] * b[0] + c[0] * d[0]) / (a[0]**2 + c[0]**2 - b[0]**2 - d[0]**2)
        theta1 = 0.5 * np.arctan(tan_theta2)
        if theta1 < 0:
            theta1 += np.pi / 2
        sin_2theta = np.sin(2 * theta1)
        cos_2theta = np.cos(2 * theta1)
        cos_theta_square = (1 + cos_2theta) / 2
        sin_theta_square = (1 - cos_2theta) / 2

        axis_theta1 = (a[0]**2 + c[0]**2) * cos_theta_square + (a[0] * b[0] + c[0] * d[0]) * sin_2theta + (b[0]**2 + d[0]**2) * sin_theta_square
        axis_theta2 = (a[0]**2 + c[0]**2) * sin_theta_square - (a[0] * b[0] + c[0] * d[0]) * sin_2theta + (b[0]**2 + d[0]**2) * cos_theta_square

        if axis_theta1 < axis_theta2:
            theta1 += np.pi / 2

        costh1 = np.cos(theta1)
        sinth1 = np.sin(theta1)
        a_star_1 = costh1 * a[0] + sinth1 * b[0]
        c_star_1 = costh1 * c[0] + sinth1 * d[0]
        psi1 = np.arctan(c_star_1 / a_star_1)
        
        if c_star_1>0 and a_star_1<0:
            psi1 = np.pi -psi1
        
        if c_star_1<0 and a_star_1<0:
            psi1 = np.pi + psi1

        if c_star_1<0 and a_star_1>0:
            psi1 = 2*np.pi - psi1



        E = np.sqrt(a_star_1**2 + c_star_1**2)

        if sc:
            a /= E
            b /= E
            c /= E
            d /= E

        cospsi1 = np.cos(psi1)
        sinpsi1 = np.sin(psi1)
        normalized_all = np.zeros((n, 4))

        if ro:
            for i in range(n):
                normalized = np.dot([[cospsi1, sinpsi1], [-sinpsi1, cospsi1]], [[a[i], b[i]], [c[i], d[i]]])
                normalized_all[i] = normalized.reshape(1, 4)
            a = normalized_all[:, 0]
            b = normalized_all[:, 1]
            c = normalized_all[:, 2]
            d = normalized_all[:, 3]

        normalized_all_1 = np.zeros((n, 4))

        if sta:
            for i in range(n):
                normalized_1 = np.dot([[a[i], b[i]], [c[i], d[i]]], [[np.cos(theta1 * (i + 1)), -np.sin(theta1 * (i + 1))], [np.sin(theta1 * (i + 1)), np.cos(theta1 * (i + 1))]])
                normalized_all_1[i, :] = normalized_1.reshape(1, 4)
            a = normalized_all_1[:, 0]
            b = normalized_all_1[:, 1]
            c = normalized_all_1[:, 2]
            d = normalized_all_1[:, 3]


        if y_sy:
            if n>1:
                if a[1] < -EPS:
                    for i in range(1, n):
                        signval = (-1) ** (((i + 1) % 2) + 1)
                        a[i] = signval * a[i]
                        d[i] = signval * d[i]
                        signval = (-1) ** ((i + 1) % 2)
                        b[i] = signval * b[i]
                        c[i] = signval * c[i]

        if x_sy:
            if n>1:
                if c[1] < -EPS:
                    b[1:] *= -1
                    c[1:] *= -1

    output = np.zeros((m, 2))

    for t in range(m):
        x_ = 0.0
        y_ = 0.0
        for i in range(n):
            x_ += (a[i] * np.cos(2 * (i + 1) * np.pi * (t) * Tk / (m - 1) / T) + b[i] * np.sin(2 * (i + 1) * np.pi * (t) * Tk / (m - 1) / T))
            y_ += (c[i] * np.cos(2 * (i + 1) * np.pi * (t) * Tk / (m - 1) / T) + d[i] * np.sin(2 * (i + 1) * np.pi * (t) * Tk / (m - 1) / T))
        output[t, 0] = A0 + x_
        output[t, 1] = C0 + y_

    return output, a, b, c, d



import numpy as np
def code2axis(chain_code, start_point):
    end_point = start_point
    axis=np.zeros((len(chain_code)+1,2))
    axis[0,:]=start_point
    i=0
    for code in chain_code:
        i=i+1
        direction = code % 8
        if direction == 0:
            end_point = np.add(end_point , [0, 1])
        elif direction == 7:
            end_point = np.add(end_point , [1, 1])
        elif direction == 6:
            end_point = np.add(end_point , [1, 0])
        elif direction == 5:
            end_point = np.add(end_point , [1, -1])
        elif direction == 4:
            end_point = np.add(end_point , [0, -1])
        elif direction == 3:
            end_point = np.add(end_point , [-1, -1])
        elif direction == 2:
            end_point = np.add(end_point , [-1, 0])
        elif direction == 1:
            end_point = np.add(end_point , [-1, 1])
        # print(end_point)
        axis[i,:]=end_point
    return axis


import numpy as np
from functions.code2axis import code2axis


def cal_area_c(chain_code,coordinates):
    # coordinates = code2axis(chain_code, [0, 0])
    # print(coordinates)
    min_y = min(coordinates[:, 1])
    max_y = max(coordinates[:, 1])
    area = 0
    for y in range(min_y, max_y + 1):
        intersection_indices = np.where(coordinates[:, 1] == y)[0]
        intersection_count = len(intersection_indices)
        intersections = np.zeros(intersection_count)
        for i, index in enumerate(intersection_indices):
            intersections[i] = coordinates[index, 0]
        intersections = np.sort(intersections)
        for i in range(0, intersection_count - 1, 2):
            area += (intersections[i+1] - intersections[i]) + 1

    print('area(pixel):', area)
    
    circumference = 0
    for code in chain_code:
        if code % 2 == 0:
            circumference += np.linalg.norm([1, 0])
        else:
            circumference += np.linalg.norm([1, 1])

    print('circumference(pixel):', circumference)
    return area, circumference





import cv2
import numpy as np

def corrosion_func(imBina, circle):
    # Create a flat disk structuring element with the specified radius
    se1 = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (2 * circle + 1, 2 * circle + 1))
    
    # Perform erosion
    im_cor = cv2.erode(imBina.astype(np.uint8), se1)
    
    return im_cor



import numpy as np
import matplotlib.pyplot as plt
from calc_traversal_dist import calc_traversal_dist

def plot_chain_code(chain_code, color='b', line_width=2):
    # Calculate traversal distance
    x_ = calc_traversal_dist(chain_code)
    
    # Starting point is assumed to be [0, 0]
    x = np.vstack(([0, 0], x_))
    plt.plot(x[:, 0], x[:, 1], color, linewidth=line_width)
    plt.show()




import numpy as np

def chain_code_func(axis_info, xidxst, yidxst, xidxed, yidxed):
    nrow, ncol = axis_info.shape
    numoftotalpoints = np.sum(axis_info == 1)
    flags = np.zeros((nrow, ncol), dtype=bool)
    xidxpre = xidxst
    yidxpre = yidxst
    contour_points = np.zeros((numoftotalpoints, 2))
    numofpoints = 1
    flags[yidxst, xidxst] = True
    contour_points[numofpoints - 1] = [yidxst, xidxst]
    chain_code = []

    while numofpoints < numoftotalpoints:
        if xidxst == xidxed and yidxst == yidxed:
            break

        yidx = np.clip(yidxst - 1, 0, nrow - 1)
        xidx = np.clip(xidxst, 0, ncol - 1)
        if axis_info[yidx, xidx] == 1 and not flags[yidx, xidx]:
            numofpoints += 1
            contour_points[numofpoints - 1] = [yidx, xidx]
            flags[yidx, xidx] = True
            yidxpre, xidxpre = yidxst, xidxst
            yidxst, xidxst = yidx, xidx
            chain_code.append(2)

        else:
            yidx = np.clip(yidxst, 0, nrow - 1)
            xidx = np.clip(xidxst + 1, 0, ncol - 1)
            if axis_info[yidx, xidx] == 1 and not flags[yidx, xidx]:
                numofpoints += 1
                contour_points[numofpoints - 1] = [yidx, xidx]
                flags[yidx, xidx] = True
                yidxpre, xidxpre = yidxst, xidxst
                yidxst, xidxst = yidx, xidx
                chain_code.append(0)

            else:
                yidx = np.clip(yidxst + 1, 0, nrow - 1)
                xidx = np.clip(xidxst, 0, ncol - 1)
                if axis_info[yidx, xidx] == 1 and not flags[yidx, xidx]:
                    numofpoints += 1
                    contour_points[numofpoints - 1] = [yidx, xidx]
                    flags[yidx, xidx] = True
                    yidxpre, xidxpre = yidxst, xidxst
                    yidxst, xidxst = yidx, xidx
                    chain_code.append(6)

                else:
                    yidx = np.clip(yidxst, 0, nrow - 1)
                    xidx = np.clip(xidxst - 1, 0, ncol - 1)
                    if axis_info[yidx, xidx] == 1 and not flags[yidx, xidx]:
                        numofpoints += 1
                        contour_points[numofpoints - 1] = [yidx, xidx]
                        flags[yidx, xidx] = True
                        yidxpre, xidxpre = yidxst, xidxst
                        yidxst, xidxst = yidx, xidx
                        chain_code.append(4)

                    else:
                        yidx = np.clip(yidxst + 1, 0, nrow - 1)
                        xidx = np.clip(xidxst + 1, 0, ncol - 1)
                        if axis_info[yidx, xidx] == 1 and not flags[yidx, xidx]:
                            numofpoints += 1
                            contour_points[numofpoints - 1] = [yidx, xidx]
                            flags[yidx, xidx] = True
                            yidxpre, xidxpre = yidxst, xidxst
                            yidxst, xidxst = yidx, xidx
                            chain_code.append(7)

                        else:
                            yidx = np.clip(yidxst - 1, 0, nrow - 1)
                            xidx = np.clip(xidxst + 1, 0, ncol - 1)
                            if axis_info[yidx, xidx] == 1 and not flags[yidx, xidx]:
                                numofpoints += 1
                                contour_points[numofpoints - 1] = [yidx, xidx]
                                flags[yidx, xidx] = True
                                yidxpre, xidxpre = yidxst, xidxst
                                yidxst, xidxst = yidx, xidx
                                chain_code.append(1)

                            else:
                                yidx = np.clip(yidxst + 1, 0, nrow - 1)
                                xidx = np.clip(xidxst - 1, 0, ncol - 1)
                                if axis_info[yidx, xidx] == 1 and not flags[yidx, xidx]:
                                    numofpoints += 1
                                    contour_points[numofpoints - 1] = [yidx, xidx]
                                    flags[yidx, xidx] = True
                                    yidxpre, xidxpre = yidxst, xidxst
                                    yidxst, xidxst = yidx, xidx
                                    chain_code.append(5)
                                else:
                                    yidx = np.clip(yidxst - 1, 0, nrow - 1)
                                    xidx = np.clip(xidxst - 1, 0, ncol - 1)
                                    if axis_info[yidx, xidx] == 1 and not flags[yidx, xidx]:
                                        numofpoints += 1
                                        contour_points[numofpoints - 1] = [yidx, xidx]
                                        flags[yidx, xidx] = True
                                        yidxpre, xidxpre = yidxst, xidxst
                                        yidxst, xidxst = yidx, xidx
                                        chain_code.append(3)
                                    else:
                                        break
    return chain_code



# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

from .build_sam import (
    build_sam,
    build_sam_vit_h,
    build_sam_vit_l,
    build_sam_vit_b,
    sam_model_registry,
)
from .predictor import SamPredictor
from .automatic_mask_generator import SamAutomaticMaskGenerator



import numpy as np
import matplotlib.pyplot as plt
from fourier_approx_modify import fourier_approx_modify

def plot_fourier_approx_modify(ai, n, m, normalized=False, color='b', line_width=2, mode=None):
    if mode is None:
        mode = 'fourier'
    # Do Fourier approximation
    x_ = fourier_approx_modify(ai, n, m, normalized, mode)

    # Make it closed contour
    x = np.vstack((x_, x_[0]))

    plt.plot(x[:, 0], x[:, 1], color, linewidth=line_width)
    plt.show()



import numpy as np

def gui_chain_code_func(axis_info, oringin_ori):
    nrow, ncol = axis_info.shape
    # print(nrow)
    # print(ncol)
    idxall = np.where(axis_info == 255)
    print(idxall)
    # idxall=np.transpose(idxall)
    # print(idxall)
    numoftotalpoints = len(idxall[0])

    rowidxst = oringin_ori[0]
    colidxst = oringin_ori[1]
    oringin_ori = [rowidxst, colidxst]
    
    flags = np.zeros((nrow, ncol))
    contour_points = np.zeros((numoftotalpoints, 2))
    # print(contour_points)
    backword_points = np.zeros((numoftotalpoints, 2))
    chain_code_ori = np.zeros(numoftotalpoints, dtype=int)
    
    numofpoints = 0
    numofpoints_pre = 0
    contour_points[numofpoints] = oringin_ori
    flags[rowidxst, colidxst] = 1
    numofbackword = 0
    backwordflag = False
    
    while numofpoints < numoftotalpoints-1:
        fatecount = 8
        # print(numofpoints)
        rowidxpre, colidxpre = contour_points[numofpoints]#前一个点的坐标
        
        rowidx = int(min(max(rowidxpre, 0), nrow - 1)) #
        colidx = int(min(max(colidxpre + 1,  0), ncol-1))
        # print(rowidx)
        # print(colidx)
        # print(rowidxst)
        # print(colidxst)
        
        if rowidx == rowidxst and colidx == colidxst and numofpoints > 1:
            numofpoints += 1
            contour_points[numofpoints] = [rowidx, colidx]
            chain_code_ori[numofpoints - 1] = 0
            break

        # print(axis_info[rowidx, colidx])
        # print(flags[rowidx, colidx])
        if axis_info[rowidx, colidx] == 255 and flags[rowidx, colidx] == 0:
            numofpoints += 1
            # print(numofpoints)
            contour_points[numofpoints] = [rowidx, colidx]
            flags[rowidx, colidx] = 1
            chain_code_ori[numofpoints - 1] = 0
        else:
            fatecount -= 1
            rowidx = int(min(max(rowidxpre - 1, 0), nrow-1))
            colidx = int(min(max(colidxpre + 1, 0), ncol-1))
            if rowidx == oringin_ori[0] and colidx == oringin_ori[1] and numofpoints > 1:
                numofpoints += 1
                contour_points[numofpoints] = [rowidx, colidx]
                chain_code_ori[numofpoints - 1] = 1
                break
            # print(rowidx)
            # print(colidx)
            # print(axis_info[colidx,rowidx])
            # print(flags[rowidx, colidx])
            if axis_info[rowidx, colidx] == 255 and flags[rowidx, colidx] == 0:

                numofpoints += 1
                contour_points[numofpoints] = [rowidx, colidx]
                flags[rowidx, colidx] = 1
                chain_code_ori[numofpoints - 1] = 1
            else:
                fatecount -= 1
                rowidx = int(min(max(rowidxpre - 1, 0), nrow - 1))
                colidx = int(min(max(colidxpre, 0), ncol - 1))
                if rowidx == rowidxst and colidx == colidxst and numofpoints > 1:
                    numofpoints += 1
                    contour_points[numofpoints] = [rowidx, colidx]
                    chain_code_ori[numofpoints - 1] = 2
                    break
                if axis_info[rowidx, colidx] == 255 and flags[rowidx, colidx] == 0:
                    numofpoints += 1
                    contour_points[numofpoints] = [rowidx, colidx]
                    flags[rowidx, colidx] = 1
                    chain_code_ori[numofpoints - 1] = 2
                else:
                    fatecount -= 1
                    rowidx = int(min(max(rowidxpre - 1, 0), nrow - 1))
                    colidx = int(min(max(colidxpre - 1, 0), ncol - 1))
                    if rowidx == rowidxst and colidx == colidxst and numofpoints > 1:
                        numofpoints += 1
                        contour_points[numofpoints] = [rowidx, colidx]
                        chain_code_ori[numofpoints - 1] = 3
                        break
                    if axis_info[rowidx, colidx] == 255 and flags[rowidx, colidx] == 0:
                        numofpoints += 1
                        contour_points[numofpoints] = [rowidx, colidx]
                        flags[rowidx, colidx] = 1
                        chain_code_ori[numofpoints - 1] = 3
                    else:
                        fatecount -= 1
                        rowidx = int(min(max(rowidxpre, 0), nrow - 1))
                        colidx = int(min(max(colidxpre - 1, 0), ncol - 1))
                        if rowidx == rowidxst and colidx == colidxst and numofpoints > 2:
                            numofpoints += 1
                            contour_points[numofpoints] = [rowidx, colidx]
                            chain_code_ori[numofpoints - 1] = 4
                            break
                        if axis_info[rowidx, colidx] == 255 and flags[rowidx, colidx] == 0:
                            numofpoints += 1
                            contour_points[numofpoints] = [rowidx, colidx]
                            flags[rowidx, colidx] = 1
                            chain_code_ori[numofpoints - 1] = 4
                        else:
                            fatecount -= 1
                            rowidx = int(min(max(rowidxpre + 1, 0), nrow - 1))
                            colidx = int(min(max(colidxpre - 1, 0), ncol - 1))
                            if rowidx == rowidxst and colidx == colidxst and numofpoints > 1:
                                numofpoints += 1
                                contour_points[numofpoints] = [rowidx, colidx]
                                chain_code_ori[numofpoints - 1] = 5
                                break
                            if axis_info[rowidx, colidx] == 255 and flags[rowidx, colidx] == 0:
                                numofpoints += 1
                                contour_points[numofpoints] = [rowidx, colidx]
                                flags[rowidx, colidx] = 1
                                chain_code_ori[numofpoints - 1] = 5
                            else:
                                fatecount -= 1
                                rowidx = int(min(max(rowidxpre + 1, 0), nrow - 1))
                                colidx = int(min(max(colidxpre, 0), ncol - 1))
                                if rowidx == rowidxst and colidx == colidxst and numofpoints > 1:
                                    numofpoints += 1
                                    contour_points[numofpoints] = [rowidx, colidx]
                                    chain_code_ori[numofpoints - 1] = 6
                                    break
                                if axis_info[rowidx, colidx] == 255 and flags[rowidx, colidx] == 0:
                                    numofpoints += 1
                                    contour_points[numofpoints] = [rowidx, colidx]
                                    flags[rowidx, colidx] = 1
                                    chain_code_ori[numofpoints - 1] = 6
                                else:
                                    fatecount -= 1
                                    rowidx = int(min(max(rowidxpre + 1, 0), nrow - 1))
                                    colidx = int(min(max(colidxpre + 1, 0), ncol - 1))
                                    if rowidx == rowidxst and colidx == colidxst and numofpoints > 1:
                                        numofpoints += 1
                                        contour_points[numofpoints] = [rowidx, colidx]
                                        chain_code_ori[numofpoints - 1] = 7
                                        break
                                    if axis_info[rowidx, colidx] == 255 and flags[rowidx, colidx] == 0:
                                        numofpoints += 1
                                        contour_points[numofpoints] = [rowidx, colidx]
                                        flags[rowidx, colidx] = 1
                                        chain_code_ori[numofpoints - 1] = 7
                                    else:
                                        fatecount -= 1
        
        if fatecount == 0:
            rowidxpre = int(contour_points[numofpoints, 0])
            colidxpre = int(contour_points[numofpoints, 1])
            # print(rowidxpre)
            # print(colidxpre)
            axis_info[rowidxpre, colidxpre] = 0
            if numofbackword == 0 and numofpoints > numofpoints_pre:
                numofpoints_pre = numofpoints
                backwordflag = 1
            numofpoints -= 1
            if numofpoints < 1:
                break
            numofbackword += 1
            if numofbackword > 20:
                break
            else:
                if backwordflag == 1:
                    backword_points[numofbackword] = [rowidxpre, colidxpre]
        else:
            numofbackword = 0
            backwordflag = False
    
    chain_code = chain_code_ori[:numofpoints]
    print(chain_code)
    # endpoint = backword_points[0] 
    oringin = oringin_ori
    print(oringin)
    # print(endpoint)
    # print((np.where(flags == 1)))
    return chain_code, oringin



def write_data_to_file(data, filename):

    # Open a text file to write data
    with open(filename, 'w') as file:
        if data.ndim>1:
            # Writes data to a text file by line
            for row in data:
                # print(row)
                file.write(' '.join(map(str, row)) + '\n')
        else:
            file.write(' '.join(map(str, data)) + '\n')
    file.close()


import numpy as np
def calc_traversal_time(ai):
    t_ = 0
    if(np.isscalar(ai)):
        t = np.zeros(1)
        t_ = t_ + 1 + ((np.sqrt(2) - 1) / 2) * (1 - (-1) ** ai)
        t[0] = t_
    else:  
        t = np.zeros(ai.shape[0])
        for i in range(ai.shape[0]):
            t_ = t_ + 1 + ((np.sqrt(2) - 1) / 2) * (1 - (-1) ** ai[i])
            t[i] = t_
    return t



import numpy as np
from calc_traversal_dist import calc_traversal_dist
from calc_traversal_time import calc_traversal_time
def calc_harmonic_coefficients_modify(ai, n, mode):
    k = ai.shape[0]
    d = calc_traversal_dist(ai)
    if mode == 0:
        edist = d[k-1, 0] ** 2 + d[k-1, 1] ** 2
        if edist > 2:
            print('error chaincode, not close form')
            return None
        else:
            if edist > 0:
                vect = [-d[k-1, 0], -d[k-1, 1]]
                if vect[0] == 1 and vect[1] == 0:
                    ai = np.append(ai, 0)
                elif vect[0] == 1 and vect[1] == 1:
                    ai = np.append(ai, 1)
                elif vect[0] == 0 and vect[1] == 1:
                    ai = np.append(ai, 2)
                elif vect[0] == -1 and vect[1] == 1:
                    ai = np.append(ai, 3)
                elif vect[0] == -1 and vect[1] == 0:
                    ai = np.append(ai, 4)
                elif vect[0] == -1 and vect[1] == -1:
                    ai = np.append(ai, 5)
                elif vect[0] == 0 and vect[1] == -1:
                    ai = np.append(ai, 6)
                elif vect[0] == 1 and vect[1] == -1:
                    ai = np.append(ai, 7)

    elif mode == 1:
        if d[k-1, 0] != 0:
            exp1 = np.zeros(abs(d[k-1, 0])) + 2 * np.sign(d[k-1, 0]) + 2 * np.sign(d[k-1, 0]) * np.sign(d[k-1, 0])
            ai = np.append(ai, exp1)
        if d[k-1, 1] != 0:
            exp2 = np.zeros(abs(d[k-1, 1])) + 2 + 2 * np.sign(d[k-1, 1]) + 2 * np.sign(d[k-1, 1]) * np.sign(d[k-1, 1])
            ai = np.append(ai, exp2)

    elif mode == 2:
        if d[k-1, 0] != 0 or d[k-1, 1] != 0:
            exp = (ai + 4) % 8
            ai = np.append(ai, exp)

    k = ai.shape[0]
    t = calc_traversal_time(ai)
    d = calc_traversal_dist(ai)
    T = t[k-1]
    two_n_pi = 2 * n * np.pi

    sigma_a = 0
    sigma_b = 0
    sigma_c = 0
    sigma_d = 0

    for p in range(k):
        if p >= 1:
            tp_prev = t[p-1]
            dp_prev = d[p-1]
        else:
            tp_prev = 0
            dp_prev = np.zeros(2)

        delta_d = calc_traversal_dist(ai[p])
        # print(delta_d)
        delta_x = delta_d[:,0]
        delta_y = delta_d[:,1]
        delta_t = calc_traversal_time(ai[p])

        q_x = delta_x / delta_t
        q_y = delta_y / delta_t

        sigma_a += two_n_pi * (d[p, 0] * np.sin(two_n_pi * t[p] / T) - dp_prev[0] * np.sin(two_n_pi * tp_prev / T)) / T
        sigma_a += q_x * (np.cos(two_n_pi * t[p] / T) - np.cos(two_n_pi * tp_prev / T))
        sigma_b -= two_n_pi * (d[p, 0] * np.cos(two_n_pi * t[p] / T) - dp_prev[0] * np.cos(two_n_pi * tp_prev / T)) / T
        sigma_b += q_x * (np.sin(two_n_pi * t[p] / T) - np.sin(two_n_pi * tp_prev / T))
        sigma_c += two_n_pi * (d[p, 1] * np.sin(two_n_pi * t[p] / T) - dp_prev[1] * np.sin(two_n_pi * tp_prev / T)) / T
        sigma_c += q_y * (np.cos(two_n_pi * t[p] / T) - np.cos(two_n_pi * tp_prev / T))
        sigma_d -= two_n_pi * (d[p, 1] * np.cos(two_n_pi * t[p] / T) - dp_prev[1] * np.cos(two_n_pi * tp_prev / T)) / T
        sigma_d += q_y * (np.sin(two_n_pi * t[p] / T) - np.sin(two_n_pi * tp_prev / T))


    if n == 0:
        print("n must be non-zero")
    
    try:
        r = T / (2 * n ** 2 * np.pi ** 2)
    except ZeroDivisionError:
        r = np.inf


    a = r * sigma_a
    b = r * sigma_b
    c = r * sigma_c
    d = r * sigma_d

    return [a, b, c, d]



import numpy as np

def is_completed_chain_code(chain_code, start_point):
    end_point = np.array(start_point)
    for direction in chain_code:
        direction = direction % 8
        if direction == 0:
            end_point += np.array([0, 1])
        elif direction == 1:
            end_point += np.array([-1, 1])
        elif direction == 2:
            end_point += np.array([-1, 0])
        elif direction == 3:
            end_point += np.array([-1, -1])
        elif direction == 4:
            end_point += np.array([0, -1])
        elif direction == 5:
            end_point += np.array([1, -1])
        elif direction == 6:
            end_point += np.array([1, 0])
        elif direction == 7:
            end_point += np.array([1, 1])
    
    distance = np.sqrt(np.sum((np.array(start_point) - end_point) ** 2))
    
    is_closed = distance <= 2
    
    if is_closed:
        print('Chain code is closed.')
    else:
        print('Chain code is not closed.')
    
    return is_closed,end_point




import numpy as np
def calc_traversal_dist(ai):
    x_ = 0
    y_ = 0
    if(np.isscalar(ai)):
        p = np.zeros((1, 2))
        x_ += np.sign(6 - ai) * np.sign(2 - ai)
        y_ += np.sign(4 - ai) * np.sign(ai)
        p[0, 0] = x_
        p[0, 1] = y_  
    else:  
        p = np.zeros((ai.shape[0], 2))
        for i in range(ai.shape[0]):
            x_ += np.sign(6 - ai[i]) * np.sign(2 - ai[i])
            y_ += np.sign(4 - ai[i]) * np.sign(ai[i])
            p[i, 0] = x_
            p[i, 1] = y_
    return p




from setuptools import setup,find_packages


setup(
    name="ElliShape",
    version="1.4.1",
    packages= find_packages(include=['ElliShape', 'ElliShape.*', 'functions', 'functions.*','segment_anything', 'segment_anything.*',]),
    python_requires=">=3.8",  # 指定 Python 版本要求
    # cmdclass={"install": CustomInstall},
    install_requires=[
        "opencv-python>=4.9.0.80",
        "matplotlib>=3.7.2",
        "pyqt5>=5.15.10",
        "segment-anything>=1.0",
        "torch>=2.2.2",
        "torchvision>=0.17.2",
        "scipy>=1.10.1",
        "openpyxl>=3.1.2",
        "numpy>=1.24.3",
        "pandas>=2.0.3",
        "pillow>=10.2.0",
        "PyQt5-sip>=12.13.0",
        "PySocks>=1.7.1",
        "urllib3>=2.1.0",
        "requests>=2.31.0",
        "tqdm>=4.66.5",
    ],
    entry_points={
        'console_scripts': [
            'ElliShape=ElliShape.ElliShape:main',
        ],
    },
)



import cv2
import numpy as np
from scipy.signal import convolve
def contour_extraction_func(graydata,mode,threshvalue1,threshvalue2):
    # Apply edge detection
    if mode==1:
        im_contour = cv2.Canny(graydata, threshvalue1,threshvalue2)  #canny
    elif mode==2: #sobel
        sobelx = cv2.Sobel(graydata, cv2.CV_64F, 1, 0, ksize=15)
        sobely = cv2.Sobel(graydata, cv2.CV_64F, 0, 1, ksize=15)
        im_contour = cv2.magnitude(sobelx, sobely)
    elif mode==3:   #zerocross
        # 定义一个零交叉核
        kernel = np.array([[-1, -1, -1],
                        [-1,  8, -1],
                        [-1, -1, -1]])

        # 使用 convolve 函数应用零交叉核
        filtered_image = convolve(graydata, kernel)
        # 生成二值图像，将零交叉点设为边缘点
        im_contour = np.zeros_like(filtered_image)
        im_contour[filtered_image > 0] = 255
    elif mode==4:#laplace
        im_contour=cv2.Laplacian(graydata, cv2.CV_64F)
    elif mode==5:
            # 使用 Roberts 算子进行边缘检测
        roberts_kernel_x = np.array([[1, 0], [0, -1]])
        roberts_kernel_y = np.array([[0, 1], [-1, 0]])
        # 对图像进行卷积
        gradient_x = cv2.filter2D(graydata, -1, roberts_kernel_x)
        gradient_y = cv2.filter2D(graydata, -1, roberts_kernel_y)
        # 计算梯度幅值
        magnitude = np.sqrt(np.square(gradient_x) + np.square(gradient_y))
        # 将梯度幅值映射到 0-255 的范围
        magnitude = (magnitude / np.max(magnitude)) * 255
        im_contour = magnitude.astype(np.uint8)
    elif mode==6:
            # 使用 Prewitt 算子进行边缘检测
        prewitt_kernel_x = np.array([[-1, 0, 1], [-1, 0, 1], [-1, 0, 1]])
        prewitt_kernel_y = np.array([[-1, -1, -1], [0, 0, 0], [1, 1, 1]])

        # 对图像进行卷积
        gradient_x = cv2.filter2D(graydata, -1, prewitt_kernel_x)
        gradient_y = cv2.filter2D(graydata, -1, prewitt_kernel_y)

        # 计算梯度幅值
        magnitude = np.sqrt(np.square(gradient_x) + np.square(gradient_y))

        # 将梯度幅值映射到 0-255 的范围
        magnitude = (magnitude / np.max(magnitude)) * 255
        im_contour = magnitude.astype(np.uint8)
    
    return im_contour



